@@ -40,7 +40,6 @@
 /* Global Variable definitions                                          */
 /*----------------------------------------------------------------------*/
 
-extern short textpos, textend;  /* keep track of the cursor position in text */
 extern Display  *dpy;    /* Works well to make this globally accessible */
 extern int *appcolors;
 extern Cursor   appcursors[NUM_CURSORS];
@@ -313,6 +312,7 @@ objinstptr new_objinst(objinstptr destinst, objinstptr srcinst, int x, int y)
 
 /*--------------------------------------------------------------*/
 /* Generic element destructor function				*/
+/* (Note---this function is not being used anywhere. . .)	*/
 /*--------------------------------------------------------------*/
 
 void remove_element(objinstptr destinst, genericptr genelem)
@@ -324,7 +324,7 @@ void remove_element(objinstptr destinst, genericptr genelem)
    destobject = locdestinst->thisobject;
 
    genelem->type &= REMOVE_TAG;
-   delete_tagged(destobject);
+   delete_tagged(locdestinst);
    calcbboxvalues(locdestinst, (genericptr *)NULL);
    updatepagebounds(destobject);
 }
@@ -456,7 +456,7 @@ void textbutton(u_char dopin, int x, int y)
    UDrawTLine(*newlabel);
    areawin->origin.x = userpt.x;
    areawin->origin.y = userpt.y;
-   textpos = 1;  /* Text position is *after* the font declaration */
+   areawin->textpos = 1;  /* Text position is *after* the font declaration */
 }
 
 /*----------------------------------------------------------------------*/
@@ -471,10 +471,10 @@ void charreport(labelptr curlabel)
    stringpart *strptr;
 
    _STR2[0] = '\0';
-   for (i = textpos - MAXCHARS; i <= textpos + MAXCHARS - 1; i++) {
+   for (i = areawin->textpos - MAXCHARS; i <= areawin->textpos + MAXCHARS - 1; i++) {
       if (i < 0) continue; 
       strptr = findstringpart(i, &locpos, curlabel->string, areawin->topinstance);
-      if (i == textpos) {
+      if (i == areawin->textpos) {
 	 strncat(_STR2, "| ", cleft);
 	 cleft -= 2;
       }
@@ -528,25 +528,27 @@ Boolean labeltext(int keypressed, char *clientdata)
 
    curlabel = TOLABEL(EDITPART);
 
-   if (curlabel == NULL || curlabel->type != LABEL || textpos <= 0) {
+   if (curlabel == NULL || curlabel->type != LABEL || areawin->textpos <= 0) {
       Wprintf("Error:  Bad label string");
+      eventmode = NORMAL_MODE;
       return FALSE;
    }
 
    /* find text segment of the current position */
-   curpos = findstringpart(textpos, &locpos, curlabel->string, areawin->topinstance);
+   curpos = findstringpart(areawin->textpos, &locpos, curlabel->string,
+		areawin->topinstance);
 
    UDrawTLine(curlabel);
 
    if (clientdata != NULL && keypressed == TEXT_DELETE) {
-      if (textpos > 1) {
+      if (areawin->textpos > 1) {
          int curloc, strpos;
          stringpart *strptr;
 
-         if (textend == 0) textend = textpos - 1;
+         if (areawin->textend == 0) areawin->textend = areawin->textpos - 1;
 
          undrawtext(curlabel);
-         for (strpos = textpos - 1; strpos >= textend; strpos--) {
+         for (strpos = areawin->textpos - 1; strpos >= areawin->textend; strpos--) {
 	    strptr = findstringpart(strpos, &curloc, curlabel->string,
 			 areawin->topinstance);
 	    if (curloc >= 0) {
@@ -564,22 +566,22 @@ Boolean labeltext(int keypressed, char *clientdata)
 	       if ((strptr->type != PARAM_START) && (strptr->type != PARAM_END))
 	          deletestring(strptr, &curlabel->string, areawin->topinstance);
 	       else
-	          textpos++;
+	          areawin->textpos++;
 	    }
 	    else
 	       Fprintf(stdout, "Error:  Unexpected NULL string part\n");
-            textpos--;
+            areawin->textpos--;
          }
-         textend = 0;
+         areawin->textend = 0;
          do_redraw = True;
       }
    }
    else if (clientdata != NULL && keypressed == TEXT_DEL_PARAM) {
-      if (textpos > 1) {
+      if (areawin->textpos > 1) {
          int curloc, strpos;
          stringpart *strptr;
 
-         strptr = findstringpart(textpos - 1, &curloc, curlabel->string,
+         strptr = findstringpart(areawin->textpos - 1, &curloc, curlabel->string,
 			 areawin->topinstance);
          if ((curloc < 0) && (strptr != NULL) && (strptr->type == PARAM_END)) {
 	    undrawtext(curlabel);
@@ -671,7 +673,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 	       undrawtext(curlabel);
 	       curlabel->string->nextpart->data.string = (char *)realloc(
 			curlabel->string->nextpart->data.string,
-		  	(strlen(_STR) + 1) * sizeof(char));
+		  	(strlen(libobj->name) + 1) * sizeof(char));
 	       strcpy(curlabel->string->nextpart->data.string, libobj->name);
                XcSetFunction(GXcopy);
 	       redrawtext(curlabel);
@@ -714,16 +716,19 @@ Boolean labeltext(int keypressed, char *clientdata)
 	 /* name is unique, change every pin in the matching symbol/	*/
 	 /* schematic to match the new text.				*/
 
-	 if ((curlabel->pin == LOCAL) && (topobject->symschem != NULL)) {
+	 if ((curlabel->pin == LOCAL) && (topobject->symschem != NULL) &&
+			(topobject->symschem->schemtype != PRIMARY)) {
 	    if ((findlabelcopy(curlabel, labelbuf) == NULL)
 			&& (findlabelcopy(curlabel, curlabel->string) == NULL)) {
-	       changeotherpins(curlabel, labelbuf);
-	       if (topobject->schemtype == PRIMARY || topobject->schemtype == SECONDARY)
-	          Wprintf("Changed corresponding pin in associated symbol");
-	       else
-	          Wprintf("Changed corresponding pin in associated schematic");
-	       incr_changes(topobject->symschem);
-	       invalidate_netlist(topobject->symschem);
+	       if (changeotherpins(curlabel, labelbuf) > 0) {
+	          if (topobject->schemtype == PRIMARY ||
+				topobject->schemtype == SECONDARY)
+	             Wprintf("Changed corresponding pin in associated symbol");
+	          else
+	             Wprintf("Changed corresponding pin in associated schematic");
+	          incr_changes(topobject->symschem);
+	          invalidate_netlist(topobject->symschem);
+	       }
 	    }
 	 }
 	
@@ -737,15 +742,15 @@ Boolean labeltext(int keypressed, char *clientdata)
       return r;
    }
    else if (clientdata != NULL && keypressed == TEXT_RIGHT) {
-      if (curpos != NULL) textpos++;
+      if (curpos != NULL) areawin->textpos++;
    }
    else if (clientdata != NULL && keypressed == TEXT_LEFT) {
-      if (textpos > 1) textpos--;
+      if (areawin->textpos > 1) areawin->textpos--;
    }
    else if (clientdata != NULL && keypressed == TEXT_DOWN) {
       while (curpos != NULL) {
-	 textpos++;
-	 curpos = findstringpart(textpos, &locpos, curlabel->string,
+	 areawin->textpos++;
+	 curpos = findstringpart(areawin->textpos, &locpos, curlabel->string,
 			areawin->topinstance);
 	 if (curpos != NULL)
 	    if (curpos->type == RETURN)
@@ -753,36 +758,38 @@ Boolean labeltext(int keypressed, char *clientdata)
       }
    }
    else if (clientdata != NULL && keypressed == TEXT_UP) {
-      while (textpos > 1) {
-	 textpos--;
-	 curpos = findstringpart(textpos, &locpos, curlabel->string,
+      while (areawin->textpos > 1) {
+	 areawin->textpos--;
+	 curpos = findstringpart(areawin->textpos, &locpos, curlabel->string,
 	 		areawin->topinstance);
 	 if (curpos->type == RETURN) {
-	    if (textpos > 1) textpos--;
+	    if (areawin->textpos > 1) areawin->textpos--;
 	    break;
 	 }
       }
    }
    else if (clientdata != NULL && keypressed == TEXT_HOME)
-      textpos = 1;
+      areawin->textpos = 1;
    else if (clientdata != NULL && keypressed == TEXT_END)
-      textpos = stringlength(curlabel->string, True, areawin->topinstance);
+      areawin->textpos = stringlength(curlabel->string, True, areawin->topinstance);
    else if (clientdata != NULL && keypressed == TEXT_SPLIT) {
       labelptr *newlabel;
       XPoint points[4], points1[4], points2[4];
 
       /* Everything after the cursor gets dumped into a new label */
 
-      if ((textpos > 1) && (curpos != NULL)) {
+      if ((areawin->textpos > 1) && (curpos != NULL)) {
 	 labelbbox(curlabel, points, areawin->topinstance);
          undrawtext(curlabel);
 	 NEW_LABEL(newlabel, topobject);
 	 labeldefaults(*newlabel, curlabel->pin, curlabel->position.x,
 		curlabel->position.y);
          if (locpos > 0)
-            curpos = splitstring(textpos, &curlabel->string, areawin->topinstance);
+            curpos = splitstring(areawin->textpos, &curlabel->string,
+			areawin->topinstance);
 	 /* move back one position to find end of top part of string */
-         curpos = splitstring(textpos - 1, &curlabel->string, areawin->topinstance);
+         curpos = splitstring(areawin->textpos - 1, &curlabel->string,
+			areawin->topinstance);
 	 if (curpos->nextpart->type == FONT_NAME) {
 	    freelabel((*newlabel)->string);
 	    (*newlabel)->string = curpos->nextpart;
@@ -820,7 +827,8 @@ Boolean labeltext(int keypressed, char *clientdata)
       /* erase first before redrawing unless the string is empty */
       undrawtext(curlabel);
       if (locpos > 0) {
-         curpos = splitstring(textpos, &curlabel->string, areawin->topinstance);
+         curpos = splitstring(areawin->textpos, &curlabel->string,
+			areawin->topinstance);
 	 curpos = curpos->nextpart;
       }
       newpart = makesegment(&curlabel->string, curpos);
@@ -849,8 +857,8 @@ Boolean labeltext(int keypressed, char *clientdata)
 	    else {
 	       /* Forward edit cursor position to the end of the parameter */
 	       do {
-	          textpos++;
-	          curpos = findstringpart(textpos, &locpos, curlabel->string,
+	          areawin->textpos++;
+	          curpos = findstringpart(areawin->textpos, &locpos, curlabel->string,
 	 		areawin->topinstance);
 	       } while (curpos->type != PARAM_END);
 	    }
@@ -862,7 +870,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 	 r = FALSE;
       }
       else {
-         textpos++;
+         areawin->textpos++;
       }
       do_redraw = True;
    }
@@ -879,8 +887,8 @@ Boolean labeltext(int keypressed, char *clientdata)
       /* Current position is not in a text string */
       if (locpos < 0) {
 
-         /* Find part of string which is immediately in front of textpos */
-         lastpos = findstringpart(textpos - 1, &locpos, curlabel->string,
+         /* Find part of string which is immediately in front of areawin->textpos */
+         lastpos = findstringpart(areawin->textpos - 1, &locpos, curlabel->string,
 		areawin->topinstance);
 
 	 /* No text on either side to attach to: make a new text segment */
@@ -906,7 +914,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 		strlen(curpos->data.string + locpos) + 1);
          *(curpos->data.string + locpos) = keypressed;
       }
-      textpos++;	/* move forward to next text position */
+      areawin->textpos++;	/* move forward to next text position */
       do_redraw = True;
       r = TRUE;
    }
@@ -917,7 +925,7 @@ Boolean labeltext(int keypressed, char *clientdata)
       short beglength;
 
       tmpext = ULength(curlabel, areawin->topinstance, curlabel->scale,
-		textpos, NULL);
+		areawin->textpos, NULL);
       beglength = tmpext.width;
 	
       tmpext = ULength(curlabel, areawin->topinstance, curlabel->scale,
@@ -947,7 +955,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 
       /* find current font and adjust menubuttons as necessary */
 
-      cfont = findcurfont(textpos, curlabel->string, areawin->topinstance);
+      cfont = findcurfont(areawin->textpos, curlabel->string, areawin->topinstance);
       if (cfont < 0) {
          Wprintf("Error:  Illegal label string");
          return r;
@@ -955,7 +963,7 @@ Boolean labeltext(int keypressed, char *clientdata)
       else
          setfontmarks(cfont, -1);
 
-      textend = 0;
+      areawin->textend = 0;
    }
    return r;
 }
@@ -978,7 +986,7 @@ void rejustify(short mode)
    labelptr curlabel = NULL;
    short    *tsel;
    short jsave;
-   Boolean changed = False;
+   Boolean preselected = False, changed = False;
    static short transjust[] = {15, 13, 12, 7, 5, 4, 3, 1, 0};
 
    if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
@@ -999,6 +1007,12 @@ void rejustify(short mode)
       setfontmarks(-1, curlabel->justify);
    }
    else {
+      if (areawin->selects == 0) {
+	 if (!checkselect(LABEL))
+	    return;
+      }
+      else preselected = TRUE;
+
       for (tsel = areawin->selectlist; tsel < areawin->selectlist +
 	      areawin->selects; tsel++) {
 	 if (SELECTTYPE(tsel) == LABEL) {
@@ -1014,7 +1028,7 @@ void rejustify(short mode)
 	    }
 	 }
       }
-      if (eventmode != MOVE_MODE && eventmode != COPY_MODE)
+      if (preselected == FALSE && eventmode != MOVE_MODE && eventmode != COPY_MODE)
 	 unselect_all();
       else
 	 draw_all_selected();
@@ -1207,6 +1221,9 @@ void trackspline(xcWidget w, caddr_t clientdata, caddr_t calldata)
    origpt = newspline->ctrl[newspline->cycle];
    newspline->ctrl[newspline->cycle] = newpos;
 
+   /* In case a constrained (parameterized) point has been changed, re-substitute */
+   exprsub(eventmode == EPATH_MODE ? TOGENERIC(EDITPART) : (genericptr)newspline);
+
    calcspline(newspline);
    UDrawEditSpline(newspline);
 
@@ -1410,7 +1427,7 @@ void boxbutton(int x, int y)
 
 void trackbox(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
-   XPoint 	newpos, upos;
+   XPoint 	newpos;
    polyptr      newbox;
    pointlist	pointptr;
 
@@ -1660,6 +1677,9 @@ void trackpoly(xcWidget w, caddr_t clientdata, caddr_t calldata)
    origpt = *curpt;
    *curpt = newpos;
 
+   /* In case a constrained (parameterized) point has been changed, re-substitute */
+   exprsub(eventmode == EPATH_MODE ? TOGENERIC(EDITPART) : (genericptr)newpoly);
+
    UDrawPolygon(newpoly);
    printpos(newpos.x, newpos.y);
    areawin->save.x = newpos.x;
@@ -1790,14 +1810,22 @@ void unjoin()
    short *selectobj;
    genericptr *genp, *newg;
    pathptr oldpath;
+   polyptr oldpoly, *newpoly;
+   Boolean preselected;
+   short i, cycle;
+
+   if (areawin->selects == 0) {
+      select_element(PATH | POLYGON);
+      preselected = FALSE;
+   }
+   else preselected = TRUE;
 
-   if (areawin->selects == 0) select_element(PATH);
    if (areawin->selects == 0) {
       Wprintf("No objects selected.");
       return;
    }
 
-   /* for each selected path */
+   /* for each selected path or polygon */
 
    XcSetFunction(GXcopy);
 
@@ -1827,8 +1855,26 @@ void unjoin()
          freepathparts(selectobj, 0);
          reviseselect(areawin->selectlist, areawin->selects, selectobj);
       }
+      else if (SELECTTYPE(selectobj) == POLYGON) {
+	 /* Method to break a polygon, in lieu of the edit-mode	*/
+	 /* polygon break that was removed.			*/
+         oldpoly = SELTOPOLY(selectobj);
+         UDrawPolygon(oldpoly);
+
+	 /* Get the point nearest the cursor, and break at that point */
+         cycle = closepoint(oldpoly, &areawin->save);
+	 if (cycle > 0 && cycle < (oldpoly->number - 1)) {
+            NEW_POLY(newpoly, topobject);
+	    topobject->parts++;
+	    polycopy(*newpoly, oldpoly);
+	    for (i = cycle; i < oldpoly->number; i++)
+	       (*newpoly)->points[i - cycle] = (*newpoly)->points[i];
+	    oldpoly->number = cycle + 1;
+	    (*newpoly)->number = (*newpoly)->number - cycle;
+	 }
+      }
    }
-   clearselects();
+   if (!preselected) clearselects();
    drawarea(NULL, NULL, NULL);
 }
 
@@ -2059,16 +2105,6 @@ void join()
       register_for_undo(XCF_Delete, UNDO_DONE, areawin->topinstance,
                 delobj, NORMAL);
 
-  //  standard_element_delete(NORMAL);
-
-  //   for (sptr = scount; sptr < scount + numpolys; sptr++) {
-  //	 easydraw(*sptr, DOFORALL);
-  //	 freepathparts(sptr, 1);
-  //	 /* revise the list of path elements */
-  //	 for (sptr2 = sptr + 1; sptr2 < scount + numpolys; sptr2++)
-  //	    if (*sptr2 > *sptr) (*sptr2)--;
-  //   }
-
       XcSetForeground((*newpoly)->color);
       UDrawPolygon(*newpoly);
 
@@ -2133,7 +2169,11 @@ void join()
    /* clean up */
 
    incr_changes(topobject);
-   clearselects();
+   /* Do not clear the selection, to be consistent with all the	*/
+   /* other actions that clear only when something has not been	*/
+   /* preselected before the action.  Elements must be selected	*/
+   /* prior to the "join" action, by necessity.			*/
+   /* clearselects(); */
    free(scount);
    free(order);
    free(direc);
