@@ -89,7 +89,7 @@ void loadlgf(int mode)
 
    /* Make sure that LGF object library has been loaded by loading it now. */
 
-   if (check_library(LGF_LIB) < 0) {
+   if (NameToLibrary(LGF_LIB) < 0) {
       int ilib;
       strcpy(_STR, LGF_LIB);
       ilib = createlibrary(FALSE);
