@@ -43,7 +43,6 @@ extern Pixmap STIPPLE[8];
 extern XCWindowData *areawin;
 extern Globaldata xobjs;
 extern int *appcolors;
-extern short textpos, textend;
 
 /*------------------------------------------------------------------------*/
 /* find the squared length of a wire (or distance between two points in   */
@@ -1161,10 +1160,9 @@ void calcextents(genericptr *bboxgen, short *llx, short *lly,
 /* Calculate the bounding box of an object instance			*/
 /*----------------------------------------------------------------------*/
 
-void objinstbbox(objinstptr obbox, XPoint *npoints, Boolean expand)
+void objinstbbox(objinstptr obbox, XPoint *npoints, int extend)
 {
    XPoint points[4];
-   int extend = (expand) ? 4 : 0;
 
    points[0].x = points[1].x = obbox->bbox.lowerleft.x - extend;
    points[1].y = points[2].y = obbox->bbox.lowerleft.y + obbox->bbox.height
@@ -1257,7 +1255,7 @@ void calcbboxsingle(genericptr *bboxgen, objinstptr thisinst,
    switch(ELEMENTTYPE(*bboxgen)) {
 
       case(OBJINST):
-	 objinstbbox(TOOBJINST(bboxgen), npoints, False);
+	 objinstbbox(TOOBJINST(bboxgen), npoints, 0);
 
          for (j = 0; j < 4; j++) {
             bboxcalc(npoints[j].x, llx, urx);
@@ -1605,7 +1603,7 @@ nextgen:
    /* with only pin labels, the bounds may not have been changed from	*/
    /* their initial values.  If so, then don't touch the bounding box.	*/
 
-   if ((llx < urx) && (lly < ury)) {
+   if ((llx <= urx) && (lly <= ury)) {
       thisobj->bbox.lowerleft.x = llx;
       thisobj->bbox.lowerleft.y = lly;
       thisobj->bbox.width = urx - llx;
@@ -1935,9 +1933,9 @@ void UDrawTextLine(labelptr curlabel, short tpos)
 
 void UDrawTLine(labelptr curlabel)
 {
-   UDrawTextLine(curlabel, textpos);
-   if ((textend > 0) && (textend < textpos)) {
-      UDrawTextLine(curlabel, textend);
+   UDrawTextLine(curlabel, areawin->textpos);
+   if ((areawin->textend > 0) && (areawin->textend < areawin->textpos)) {
+      UDrawTextLine(curlabel, areawin->textend);
    }
 }
 
@@ -1996,65 +1994,122 @@ void UDrawBox(XPoint origin, XPoint corner)
 /* closely reach the position "corner".					*/
 /*----------------------------------------------------------------------*/
 
-void UDrawRescaleBox(XPoint *corner)
+float UDrawRescaleBox(XPoint *corner)
 {
    XPoint 	origpoints[5], newpoints[5];
    genericptr	rgen;
-   float	savescale, newscale, lastscale, tempscale;
-   long		mindist, testdist, lastmindist;
+   float	savescale, newscale;
+   long		mindist, testdist, refdist;
    labelptr	rlab;
-   int		i, j; /* , idx; (jdk) */
+   graphicptr   rgraph;
+   objinstptr	rinst;
+   int		i;
 
-   /* To be done---draw a box around each selected part, not	*/
-   /* just the first one.					*/
+   if (areawin->selects == 0) return 0.0;
 
    SetFunction(dpy, areawin->gc, GXxor);
    SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
    SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapRound, JoinBevel);
    
-   for (j = 0; j < areawin->selects; j++) {
-      rgen = SELTOGENERIC(areawin->selectlist + j);
+   /* Use only the 1st selection as a reference to set the scale */
 
-      switch(ELEMENTTYPE(rgen)) {
-         case LABEL:
-	    rlab = (labelptr)rgen;
-	    savescale = rlab->scale;
-	    newscale = lastscale = rlab->scale * 2;
-	    mindist = LONG_MAX;
-	    while (mindist > 4) {
-	       rlab->scale = newscale;
-	       labelbbox(rlab, newpoints, areawin->topinstance);
-	       newpoints[5] = newpoints[0];
-	       lastmindist = mindist;
-	       mindist = LONG_MAX;
-	       for (i = 0; i < 4; i++) {
-	          testdist = finddist(&newpoints[i], &newpoints[i+1], corner);
-	          if (testdist < mindist)
-	             mindist = testdist;
-	       }
-	       if (mindist == lastmindist)
-		  break;
-	       else if (lastmindist == LONG_MAX)
-		  newscale /= 2;
-	       else {
-	          tempscale = (fabs)((newscale * lastmindist) - (lastscale * mindist)) / 
-				(abs)(lastmindist - mindist);
-	          lastscale = newscale;
-	          newscale = tempscale;
-	       }
-	    }
-	    rlab->scale = savescale;
-            break;
-         case GRAPHIC:
-            break;
-         case OBJINST:
-            break;
-         /* Note:  should be able to handle other elements, as well */
-      }
+   rgen = SELTOGENERIC(areawin->selectlist);
 
-      UTransformbyCTM(DCTM, newpoints, origpoints, 4);
-      strokepath(origpoints, 4, 0, 1);
+   switch(ELEMENTTYPE(rgen)) {
+      case LABEL:
+	 rlab = (labelptr)rgen;
+	 labelbbox(rlab, newpoints, areawin->topinstance);
+	 newpoints[4] = newpoints[0];
+	 mindist = LONG_MAX;
+	 for (i = 0; i < 4; i++) {
+	    testdist = finddist(&newpoints[i], &newpoints[i+1], corner);
+	    if (testdist < mindist)
+	       mindist = testdist;
+	 }
+	 refdist = wirelength(corner, &(rlab->position));
+	 mindist = (int)sqrt(abs((double)mindist));
+	 savescale = rlab->scale;
+	 if (!test_insideness((int)corner->x, (int)corner->y, newpoints))
+	    mindist = -mindist;
+	 if (refdist == mindist) refdist = 1 - mindist;
+	 newscale = fabs(rlab->scale * (float)refdist / (float)(refdist + mindist));
+	 if (newscale > 10 * rlab->scale) newscale = 10 * rlab->scale;
+	 if (areawin->snapto) {
+	    float snapstep = 2 * (float)xobjs.pagelist[areawin->page]->gridspace
+			/ (float)xobjs.pagelist[areawin->page]->snapspace;
+	    newscale = (float)((int)(newscale * snapstep)) / snapstep;
+	    if (newscale < (1.0 / snapstep)) newscale = (1.0 / snapstep);
+	 }
+	 else if (newscale < 0.1 * rgraph->scale) newscale = 0.1 * rgraph->scale;
+	 rlab->scale = newscale;
+	 labelbbox(rlab, newpoints, areawin->topinstance);
+	 rlab->scale = savescale;
+         break;
+
+      case GRAPHIC:
+	 rgraph = (graphicptr)rgen;
+	 graphicbbox(rgraph, newpoints);
+	 newpoints[4] = newpoints[0];
+	 mindist = LONG_MAX;
+	 for (i = 0; i < 4; i++) {
+	    testdist = finddist(&newpoints[i], &newpoints[i+1], corner);
+	    if (testdist < mindist)
+	       mindist = testdist;
+	 }
+	 refdist = wirelength(corner, &(rgraph->position));
+	 mindist = (int)sqrt(abs((double)mindist));
+	 savescale = rgraph->scale;
+	 if (!test_insideness((int)corner->x, (int)corner->y, newpoints))
+	    mindist = -mindist;
+	 if (refdist == mindist) refdist = 1 - mindist;  /* avoid inf result */
+	 newscale = fabs(rgraph->scale * (float)refdist / (float)(refdist + mindist));
+	 if (newscale > 10 * rgraph->scale) newscale = 10 * rgraph->scale;
+	 if (areawin->snapto) {
+	    float snapstep = 2 * (float)xobjs.pagelist[areawin->page]->gridspace
+			/ (float)xobjs.pagelist[areawin->page]->snapspace;
+	    newscale = (float)((int)(newscale * snapstep)) / snapstep;
+	    if (newscale < (1.0 / snapstep)) newscale = (1.0 / snapstep);
+	 }
+	 else if (newscale < 0.1 * rgraph->scale) newscale = 0.1 * rgraph->scale;
+	 rgraph->scale = newscale;
+	 graphicbbox(rgraph, newpoints);
+	 rgraph->scale = savescale;
+         break;
+
+      case OBJINST:
+	 rinst = (objinstptr)rgen;
+	 objinstbbox(rinst, newpoints, 0);
+	 newpoints[4] = newpoints[0];
+	 mindist = LONG_MAX;
+	 for (i = 0; i < 4; i++) {
+	    testdist = finddist(&newpoints[i], &newpoints[i+1], corner);
+	    if (testdist < mindist)
+	       mindist = testdist;
+	 }
+	 refdist = wirelength(corner, &(rinst->position));
+	 mindist = (int)sqrt(abs((double)mindist));
+	 savescale = rinst->scale;
+	 if (!test_insideness((int)corner->x, (int)corner->y, newpoints))
+	    mindist = -mindist;
+	 if (refdist == mindist) refdist = 1 - mindist;  /* avoid inf result */
+	 newscale = fabs(rinst->scale * (float)refdist / (float)(refdist + mindist));
+	 if (newscale > 10 * rinst->scale) newscale = 10 * rinst->scale;
+	 if (areawin->snapto) {
+	    float snapstep = 2 * (float)xobjs.pagelist[areawin->page]->gridspace
+			/ (float)xobjs.pagelist[areawin->page]->snapspace;
+	    newscale = (float)((int)(newscale * snapstep)) / snapstep;
+	    if (newscale < (1.0 / snapstep)) newscale = (1.0 / snapstep);
+	 }
+	 else if (newscale < 0.1 * rgraph->scale) newscale = 0.1 * rgraph->scale;
+	 rinst->scale = newscale;
+	 objinstbbox(rinst, newpoints, 0);
+	 rinst->scale = savescale;
+         break;
    }
+
+   UTransformbyCTM(DCTM, newpoints, origpoints, 4);
+   strokepath(origpoints, 4, 0, 1);
+   return newscale;
 }
 
 /*-------------------------------------------------------------------------*/
@@ -2368,7 +2423,7 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 /*----------------------------------------------------------------------*/
 
 void UDoLatex(objinstptr theinstance, short level, FILE *f,
-	float scale, int tx, int ty, Boolean *checkonly)
+	float scale, float scale2, int tx, int ty, Boolean *checkonly)
 {
    XPoint	lpos, xlpos;
    XfPoint	xfpos;
@@ -2393,7 +2448,7 @@ void UDoLatex(objinstptr theinstance, short level, FILE *f,
 
       switch(ELEMENTTYPE(*areagen)) {
          case(OBJINST):
-            UDoLatex(TOOBJINST(areagen), level + 1, f, scale, tx, ty, checkonly);
+            UDoLatex(TOOBJINST(areagen), level + 1, f, scale, scale2, tx, ty, checkonly);
 	    break;
    
 	 case(LABEL): 
@@ -2419,11 +2474,19 @@ void UDoLatex(objinstptr theinstance, short level, FILE *f,
 		       xfpos.y -= 1.0;
 		       xfpos.x += 0.056;
 		       xfpos.y += 0.056;
+		       xfpos.x /= scale2;
+		       xfpos.y /= scale2;
 		       ltext = textprint(thislabel->string, theinstance);
 		       tbjust = thislabel->justify & (NOTBOTTOM | TOP);
 		       lrjust = thislabel->justify & (NOTLEFT | RIGHT);
-		       fprintf(f, "\\putbox{%3.2fin}{%3.2fin}{",
-				xfpos.x, xfpos.y);
+
+		       /* The 1.2 factor accounts for the difference between	 */
+		       /* Xcircuit's label scale of "1" and LaTeX's "normalsize" */
+
+		       fprintf(f, "   \\putbox{%3.2fin}{%3.2fin}{%3.2f}{",
+				xfpos.x, xfpos.y, 1.2 * thislabel->scale);
+		       if (thislabel->rotation != 0)
+			  fprintf(f, "\\rotatebox{-%d}{", thislabel->rotation);
 		       if (lrjust == (NOTLEFT | RIGHT)) fprintf(f, "\\rightbox{");
 		       else if (lrjust == NOTLEFT) fprintf(f, "\\centbox{");
 		       if (tbjust == (NOTBOTTOM | TOP)) fprintf(f, "\\topbox{");
@@ -2431,6 +2494,7 @@ void UDoLatex(objinstptr theinstance, short level, FILE *f,
 		       fprintf(f, "%s", ltext);
 		       if (lrjust != NORMAL) fprintf(f, "}");
 		       if (tbjust != NORMAL) fprintf(f, "}");
+		       if (thislabel->rotation != 0) fprintf(f, "}");
 		       fprintf(f, "}%%\n");
 		       free(ltext);
 		    }
@@ -2448,13 +2512,13 @@ void UDoLatex(objinstptr theinstance, short level, FILE *f,
 void TopDoLatex()
 {
    FILE *f;
-   float psscale;
+   float psscale, outscale;
    int tx, ty, width, height;
    XPoint origin;
    Boolean checklatex = FALSE;
    char filename[100], extend[10], *dotptr;
 
-   UDoLatex(areawin->topinstance, 0, NULL, 1.0, 0, 0, &checklatex);
+   UDoLatex(areawin->topinstance, 0, NULL, 1.0, 1.0, 0, 0, &checklatex);
 
    if (checklatex == FALSE) return;	/* No LaTeX labels to write */
 
@@ -2480,19 +2544,20 @@ void TopDoLatex()
 
    fprintf(f, "%% XCircuit output \"%s.tex\" for LaTeX input from %s%s\n",
 		filename, filename, extend);
-   fprintf(f, "\\def\\putbox#1#2#3{\\makebox[0in][l]{\\makebox[#1][l]{}"
+   fprintf(f, "\\def\\putbox#1#2#3#4{\\makebox[0in][l]{\\makebox[#1][l]{}"
 		"\\raisebox{\\baselineskip}[0in][0in]"
-		"{\\raisebox{#2}[0in][0in]{#3}}}}\n");
+		"{\\raisebox{#2}[0in][0in]{\\scalebox{#3}{#4}}}}}\n");
    fprintf(f, "\\def\\rightbox#1{\\makebox[0in][r]{#1}}\n");
    fprintf(f, "\\def\\centbox#1{\\makebox[0in]{#1}}\n");
-   fprintf(f, "\\def\\topbox#1{\\raisebox{-\\baselineskip}[0in][0in]{#1}}\n");
-   fprintf(f, "\\def\\midbox#1{\\raisebox{-0.5\\baselineskip}[0in][0in]{#1}}\n");
-   fprintf(f, "\\begin{flushleft}\n");
-   
-   fprintf(f, "\\epsfig{file=%s%s}\\\\\n", filename, extend);
+   fprintf(f, "\\def\\topbox#1{\\raisebox{-0.60\\baselineskip}[0in][0in]{#1}}\n");
+   fprintf(f, "\\def\\midbox#1{\\raisebox{-0.20\\baselineskip}[0in][0in]{#1}}\n");
 
-   psscale = getpsscale(xobjs.pagelist[areawin->page]->outscale,
-		areawin->page);
+   /* Modified to use \scalebox and \parbox by Alex Tercete, June 2008 */
+
+   fprintf(f, "\\begin{center}\n");
+
+   outscale = xobjs.pagelist[areawin->page]->outscale;
+   psscale = getpsscale(outscale, areawin->page);
 
    width = toplevelwidth(areawin->topinstance, &origin.x);
    height = toplevelheight(areawin->topinstance, &origin.y);
@@ -2500,14 +2565,23 @@ void TopDoLatex()
    tx = (int)(72 / psscale) - origin.x,
    ty = (int)(72 / psscale) - origin.y;
 
-   fprintf(f, "%% translate x=%d y=%d scale %3.2f\n", tx, ty, psscale);
+   fprintf(f, "   \\scalebox{%g}{%\n", outscale);
+   fprintf(f, "   \\normalsize\n");
+   fprintf(f, "   \\parbox{%gin}{%\n", (((float)width * psscale) / 72.0) / outscale);
+   fprintf(f, "   \\includegraphics[scale=%g]{%s%s}\\\\\n", 1.0 / outscale,
+			filename, extend);
+   fprintf(f, "   %% translate x=%d y=%d scale %3.2f\n", tx, ty, psscale);
 
    UPushCTM();		/* Save current state */
    UResetCTM(DCTM);	/* Set to identity matrix */
-   UDoLatex(areawin->topinstance, 0, f, psscale, tx, ty, NULL);
+   UDoLatex(areawin->topinstance, 0, f, psscale, outscale, tx, ty, NULL);
    UPopCTM();		/* Restore state */
 
-   fprintf(f, "\\end{flushleft}\n");
+   fprintf(f, "   } %% close \'parbox\'\n");
+   fprintf(f, "   } %% close \'scalebox\'\n");
+   fprintf(f, "   \\vspace{-\\baselineskip} %% this is not"
+		" necessary, but looks better\n");
+   fprintf(f, "\\end{center}\n");
    fclose(f);
 
    Wprintf("Wrote auxiliary file %s.tex", filename);
