@@ -49,7 +49,7 @@ keybinding *keylist = NULL;
 
 extern Display *dpy;
 extern char  _STR[150], _STR2[250];
-extern Boolean pressmode;
+extern int pressmode;
 extern XCWindowData *areawin;
 
 /*----------------------------------------------------------------------*/
@@ -101,10 +101,9 @@ static char *function_names[NUM_FUNCTIONS + 1] = {
    "Undo", "Redo", "Select Save", "Unselect", "Dashed", "Dotted",
    "Solid", "Prompt", "Dot", "Wire", "Cancel", "Nothing", "Exit",
    "Netlist", "Swap", "Pin Label", "Pin Global", "Info Label", "Graphic",
-   "Rescale", "Select Box", "Connectivity", "Continue Copy",
-   "Continue Element", "Finish Copy", "Finish Element", "Finish",
-   "Cancel Last", "Sim", "SPICE", "PCB", "SPICE Flat", "Color",
-   "Text Delete Param", "Scale",
+   "Select Box", "Connectivity", "Continue Element", "Finish Element",
+   "Continue Copy", "Finish Copy", "Finish", "Cancel Last", "Sim",
+   "SPICE", "PCB", "SPICE Flat", "Rescale", "Text Delete Param",
    NULL			/* sentinel */
 };
 
@@ -780,7 +779,7 @@ void default_keybindings()
    /* 2-button vs. 3-button mouse bindings (set with -2	*/
    /* commandline option; 3-button bindings default)	*/
 
-   if (pressmode) {
+   if (pressmode == 1) {
       add_binding(ALL_WINDOWS, BUTTON3, XCF_Text_Return);
       add_binding(ALL_WINDOWS, BUTTON3, XCF_Select);
       add_binding(ALL_WINDOWS, HOLD | BUTTON3, XCF_SelectBox);
