@@ -56,13 +56,24 @@ proc xcircuit::getsubckttext {name} {
 proc xcircuit::makesymbol {{filename ""} {orderedpins ""}} {
    global XCOps
 
+   config suspend true	;# suspend graphics and change count
+
+   set techname [.dialog.techself.techselect cget -text]
    set symbolname [.dialog.textent.txt get]
 
+   if {[string first :: $symbolname] >= 0} {	;# symbolname has tech name embedded
+      set techname ""
+   } elseif {[string first ( $techname] >= 0} {	;# "(user)" specified
+      set techname "::"
+   } elseif {[string first :: $techname] < 0} {
+      set techname "${techname}::"
+   }
+
    # If "filename" is specified then we have a netlist, not a schematic.
    # Therefore, create an object but don't use xcircuit::symbol
 
    if {$filename == ""} {
-      xcircuit::symbol make $symbolname $XCOps(library);
+      xcircuit::symbol make ${techname}${symbolname} $XCOps(library);
       set schematicname [schematic get]
       set noschem 0
    } else {
@@ -106,7 +117,7 @@ proc xcircuit::makesymbol {{filename ""} {orderedpins ""}} {
 
    if {$noschem == 1} {
       select $sbox
-      set handle [object make $symbolname $XCOps(library)]
+      set handle [object make ${techname}${symbolname} $XCOps(library)]
       push $handle
    }
 
@@ -126,6 +137,7 @@ proc xcircuit::makesymbol {{filename ""} {orderedpins ""}} {
       label $nlab type normal
       label $nlab justify right
       incr y $pinspace
+      deselect selected
    }
    set x [expr -$hwidth - $qtrspace]
    set y [expr -($leftpins - 1) * $halfspace]
@@ -141,6 +153,7 @@ proc xcircuit::makesymbol {{filename ""} {orderedpins ""}} {
       label $nlab type normal
       label $nlab justify left
       incr y $pinspace
+      deselect selected
    }
    set y [expr -$hheight -$qtrspace]
    set x [expr -($botpins - 1) * $halfspace]
@@ -157,6 +170,7 @@ proc xcircuit::makesymbol {{filename ""} {orderedpins ""}} {
       label $nlab type normal
       label $nlab justify left
       incr x $pinspace
+      deselect selected
    }
    set y [expr $hheight + $qtrspace]
    set x [expr -($toppins - 1) * $halfspace]
@@ -173,16 +187,17 @@ proc xcircuit::makesymbol {{filename ""} {orderedpins ""}} {
       label $nlab type normal
       label $nlab justify left
       incr x $pinspace
+      deselect selected
    }
 
-   deselect
+   deselect selected
    set nlab [label make "$symbolname" {0 0}]
    label $nlab justify middle
    label $nlab justify center
    element $nlab color set blue
 
-   deselect
-   parameter make substring idx "?"
+   deselect selected
+   parameter make substring index "?"
    parameter make substring class "X"
    if {$schematicname == $symbolname} {
       parameter make substring link "%n"
@@ -190,10 +205,10 @@ proc xcircuit::makesymbol {{filename ""} {orderedpins ""}} {
       parameter make substring link "$schematicname"
    }
 
-   set nlab [label make "{Parameter class} {Parameter idx}" "0 -$pinspace"]
+   set nlab [label make "{Parameter class} {Parameter index}" "0 -$pinspace"]
    label $nlab justify center
    element $nlab color set blue
-   deselect
+   deselect selected
 
    # Determine if the schematic already has a "subckt" line.  If so,
    # attempt to arrange the pin ordering from it.  If not, create one.
@@ -206,9 +221,17 @@ proc xcircuit::makesymbol {{filename ""} {orderedpins ""}} {
 
    if {$subckttext == ""} {
       set subckttext [list {Text "spice1:.subckt %n"}]
+      set pstring ""
       foreach j $pinlabels {
-         lappend subckttext "$j"
+	 if {[string length $pstring] > 60} {
+            lappend subckttext [subst {Text "$pstring"}]
+	    lappend subckttext {Return}
+	    set pstring "+"
+	 }
+	 set pstring [join [list $pstring "$j"]]   ;# preserves whitespace
       }
+      lappend subckttext [subst {Text "$pstring"}]
+
       if {$noschem == 0} {
          schematic goto
          set bbox [join [page bbox all]] 
@@ -216,12 +239,13 @@ proc xcircuit::makesymbol {{filename ""} {orderedpins ""}} {
          set y [expr [lindex $bbox 1] - $pinspace]
          set nlab [label make info "$subckttext" "$x $y"] 
          label $nlab justify center
+         deselect selected
          set y [expr $y - $pinspace]
          set nlab [label make info "spice-1:.ends" "$x $y"] 
          symbol goto
       }
 
-      set itext [list {Text "spice:"} {Parameter class} {Parameter idx}]
+      set itext [list {Text "spice:"} {Parameter class} {Parameter index}]
       set pstring ""
       foreach j $pinlabels {
 	 if {[string length $pstring] > 60} {
@@ -234,7 +258,7 @@ proc xcircuit::makesymbol {{filename ""} {orderedpins ""}} {
       set pstring [join [list $pstring "%n"]]   ;# preserves whitespace
       lappend itext [subst {Text "$pstring"}]
    } else {
-      set itext [list {Text "spice:"} {Parameter class} {Parameter idx}]
+      set itext [list {Text "spice:"} {Parameter class} {Parameter index}]
       set pstring ""
       foreach j [lrange [lindex $subckttext 0] 2 end] {
 	 if {[string length $pstring] > 60} {
@@ -250,12 +274,12 @@ proc xcircuit::makesymbol {{filename ""} {orderedpins ""}} {
 
    set y [expr -$hheight - 3 * $pinspace]
 
-   deselect
+   deselect selected
    set nlab [label make info "$itext" "0 $y"] 
    label $nlab justify center
 
    if {$noschem == 1} {
-      deselect
+      deselect selected
       if {[string index $filename 0] != "/"} {
 	 set filename [join [concat [pwd] $filename] "/"]
       }
@@ -264,14 +288,16 @@ proc xcircuit::makesymbol {{filename ""} {orderedpins ""}} {
       lappend itext [subst {Text "$filename"}]
       set nlab [label make info "$itext" "0 $y"]
       label $nlab justify center
-      deselect
+      deselect selected
       pop
    } else {
       library $XCOps(library) compose
-      deselect
+      deselect selected
       symbol goto
       zoom view
    }
+
+   config suspend false 	;# unlocked state
 }
 
 #---------------------------------------------------------------------------
@@ -288,7 +314,7 @@ proc xcircuit::getpinlist {schematicname} {
 
    if {$subckttext == {}} {
       set pinlist {}
-      deselect
+      deselect selected
       set objlist [object parts]
       
       foreach j $objlist {
@@ -296,7 +322,14 @@ proc xcircuit::getpinlist {schematicname} {
          if {$etype == "Label"} {
 	    set ltype [label $j type]
 	    if {$ltype == "local" || $ltype == "global"} {
-	       lappend pinlist [label $j text]
+
+	       # Avoid netlist-generated pins by rejecting labels that
+	       # don't start with a font specifier.
+
+	       set subtype [lindex [lindex [lindex [label $j list] 0] 0] 0]
+	       if {$subtype == "Font"} {
+	          lappend pinlist [label $j text]
+	       }
 	    }
 	 }
       }
@@ -304,7 +337,7 @@ proc xcircuit::getpinlist {schematicname} {
    } else {
       set pinlist [lrange [lindex $subckttext 0] 2 end]
    }
-   deselect
+   deselect selected
    return $pinlist
 }
 
@@ -553,6 +586,7 @@ proc xcircuit::promptmakesymbol {{name ""}} {
           wm withdraw .dialog}
   .dialog.textent.title.field configure -text "Confirm symbol name:"
   .dialog.textent.txt delete 0 end
+  .dialog.textent.txt insert 0 $name
 
   xcircuit::popupdialog
   xcircuit::addtechlist .dialog "Technology: "
