@@ -53,7 +53,6 @@ extern ApplicationData appdata;
 extern Cursor	  appcursors[NUM_CURSORS];
 extern fontinfo *fonts;
 extern short fontcount;
-extern short textpos, textend;
 
 #ifdef TCL_WRAPPER
 extern Tcl_Interp *xcinterp;
@@ -188,12 +187,13 @@ void changetextscale(float newscale)
 
    if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       settext = *((labelptr *)EDITPART);
-      if (textpos > 0 || textpos < stringlength(settext->string, True,
+      if (areawin->textpos > 0 || areawin->textpos < stringlength(settext->string, True,
 		areawin->topinstance)) {
 	 undrawtext(settext);
-	 strptr = findstringpart(textpos - 1, NULL, settext->string,
+	 strptr = findstringpart(areawin->textpos - 1, NULL, settext->string,
+			areawin->topinstance);
+	 nextptr = findstringpart(areawin->textpos, NULL, settext->string,
 			areawin->topinstance);
-	 nextptr = findstringpart(textpos, NULL, settext->string, areawin->topinstance);
 	 if (strptr->type == FONT_SCALE)
 	    strptr->data.scale = newscale;
 	 else if (nextptr && nextptr->type == FONT_SCALE)
@@ -220,7 +220,7 @@ void changetextscale(float newscale)
 	       undrawtext(settext);
                settext->scale = newscale;
 	       redrawtext(settext);
-	       register_for_undo(XCF_Scale, UNDO_MORE, areawin->topinstance,
+	       register_for_undo(XCF_Rescale, UNDO_MORE, areawin->topinstance,
 			(genericptr)settext, (double)oldscale);
 	    }
 	 }
@@ -384,12 +384,12 @@ labelptr gettextsize(float **floatptr)
    if (floatptr) *floatptr = &areawin->textscale;
 
    if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
-      if (textpos > 0 || textpos < stringlength(settext->string, True,
-		areawin->topinstance)) {
+      if (areawin->textpos > 0 || areawin->textpos < stringlength(settext->string,
+		True, areawin->topinstance)) {
          settext = *((labelptr *)EDITPART);
-	 strptr = findstringpart(textpos - 1, NULL, settext->string,
+	 strptr = findstringpart(areawin->textpos - 1, NULL, settext->string,
 			areawin->topinstance);
-	 nextptr = findstringpart(textpos, NULL, settext->string,
+	 nextptr = findstringpart(areawin->textpos, NULL, settext->string,
 			areawin->topinstance);
 	 if (strptr->type == FONT_SCALE) {
 	    if (floatptr) *floatptr = &strptr->data.scale;
@@ -488,7 +488,7 @@ void setosize(xcWidget w, objinstptr dataptr)
          nsobj->scale = tmpres;
 
          if (oldsize != tmpres)
-            register_for_undo(XCF_Scale, UNDO_MORE, areawin->topinstance,
+            register_for_undo(XCF_Rescale, UNDO_MORE, areawin->topinstance,
 			SELTOGENERIC(osel), (double)oldsize);
       }
    }
@@ -543,7 +543,7 @@ void setwwidth(xcWidget w, void *dataptr)
          }
 
 	 if (oldwidth != tmpres)
-            register_for_undo(XCF_Scale, UNDO_MORE, areawin->topinstance,
+            register_for_undo(XCF_Rescale, UNDO_MORE, areawin->topinstance,
 			SELTOGENERIC(osel), (double)oldwidth);
       }
       unselect_all();
@@ -571,16 +571,6 @@ void makenewfontbutton()
    fontnumbers[nfontnumbers - 1] = fontcount - 1;
 }
 
-/*--------------------------------------------------------------*/
-/* Set the menu checkmarks on the font menu			*/
-/*--------------------------------------------------------------*/
-
-void togglefontmark(int fontval)
-{
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontfamily",
-		fonts[fontval].family, 0);
-}
-
 #endif /* TCL_WRAPPER */
 
 /*---------------------------------------------------------------*/
@@ -635,15 +625,10 @@ void togglegrid(u_short type)
       "decimal inches",
       "fractional inches",
       "centimeters",
-      "internal units", NULL      /* (jdk) */
+      "internal units", NULL
    };
 
-   /* Note that Tcl_SetVar() doesn't require double-quotes around the	*/
-   /* string; doing so prevents the menu from recognizing the variable	*/
-   /* value.								*/
-
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),gridstyle",
-	stylenames[type], 0);
+   XcInternalTagCall(xcinterp, 3, "config", "coordstyle", stylenames[type]);
 }
 
 #endif /* TCL_WRAPPER */
@@ -811,7 +796,6 @@ void makepagebutton()
 
 void renamepage(short pagenumber)
 {
-   int page;
    objinstptr thisinst = xobjs.pagelist[pagenumber]->pageinst;
    char *pname, *plabel;
 
@@ -846,23 +830,22 @@ void setcolormark(int colorval)
 {
    /* Set GUI variables and execute any command tags associated */
    /* with the "color" command */
-
-   Tcl_Obj *objv[3];
+  
+   char cstr[5];
    int i;
 
-   objv[0] = Tcl_NewStringObj("color", 5);
-   objv[1] = Tcl_NewStringObj("set", 3);
-   if (colorval == DEFAULTCOLOR)
-      objv[2] = Tcl_NewStringObj("inherit", 7);
-   else {
+   if (colorval != DEFAULTCOLOR) {
       for (i = 0; i < number_colors; i++)
-         if (colorlist[i].color.pixel == colorval)
+         if (colorlist[i].color.pixel == colorval) {
+	    sprintf(cstr, "%5d", i);
 	    break;
-      objv[2] = Tcl_NewIntObj((int)i);
+	 }
    }
-   Tcl_SetVar2Ex(xcinterp, "XCWinOps", "$XCOps(focus),colorval", objv[2], 0);
 
-   XcTagCallback(xcinterp, 3, objv);
+#ifdef TCL_WRAPPER
+   XcInternalTagCall(xcinterp, 3, "color", "set", (colorval == DEFAULTCOLOR) ?
+	"inherit" : cstr);
+#endif
 }
 
 /*----------------------------------------------------------------*/
@@ -871,40 +854,25 @@ void setcolormark(int colorval)
 
 void setallstylemarks(u_short styleval)
 {
-   /* Set GUI variables and execute any command tags associated	*/
-   /* with the "fill" and "border" commands.			*/
+   /* Execute any command tags associated	*/
+   /* with the "fill" and "border" commands.	*/
 
-   Tcl_Obj *objv[3];
+   char fstr[10];
    int fillfactor;
    const char *bptr;
 
-   const char *borders[] = {"solid", "unbordered", "dashed", "dotted", NULL}; /* (jdk) */
+   const char *borders[] = {"solid", "unbordered", "dashed", "dotted", NULL};
    enum BorderIdx { SolidIdx, UnborderedIdx, DashedIdx, DottedIdx };
 
-   objv[0] = Tcl_NewStringObj("fill", 4);
-
    if (styleval & FILLED) {
       fillfactor = (int)(12.5 * (float)(1 + ((styleval & FILLSOLID) >> 5)));
-      if (fillfactor == 100)
-         objv[1] = Tcl_NewStringObj("solid", 5);
+      if (fillfactor < 100)
+	 sprintf(fstr, "%d", fillfactor);
       else
-         objv[1] = Tcl_NewIntObj(fillfactor);
+	 strcpy(fstr, "solid");
    }
    else
-      objv[1] = Tcl_NewStringObj("unfilled", 8);
-
-   if (styleval & OPAQUE)
-      objv[2] = Tcl_NewStringObj("opaque", 6);
-   else
-      objv[2] = Tcl_NewStringObj("transparent", 11);
-
-   Tcl_SetVar2Ex(xcinterp, "XCWinOps", "$XCOps(focus),fillamount", objv[1], 0);
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),opaque", (styleval & OPAQUE) ?
-		"true" : "false", 0);
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),bboxtype", (styleval & BBOX) ?
-		"true" : "false", 0);
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),polyclosed",
-		(styleval & UNCLOSED) ? "false" : "true", 0);
+      strcpy(fstr, "unfilled");
 
    switch (styleval & BORDERS) {
       case DASHED:
@@ -920,9 +888,14 @@ void setallstylemarks(u_short styleval)
 	 bptr = borders[SolidIdx];
 	 break;
    }
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),linestyle", (char *)bptr, 0);
 
-   XcTagCallback(xcinterp, 3, objv);
+   XcInternalTagCall(xcinterp, 3, "fill", fstr,
+		(styleval & OPAQUE) ? "opaque" : "transparent");
+   XcInternalTagCall(xcinterp, 3, "border", "bbox", (styleval & BBOX) ? "true" :
+		"false");
+   XcInternalTagCall(xcinterp, 2, "border", (styleval & UNCLOSED) ? "unclosed" :
+		"closed");
+   XcInternalTagCall(xcinterp, 2, "border", bptr);
 }
 
 #endif /* TCL_WRAPPER */
@@ -951,10 +924,19 @@ polyptr checkforbbox(objectptr localdata)
 
 int setelementstyle(xcWidget w, u_short value, u_short mask)
 {
-   Boolean selected = False;
+   Boolean preselected, selected = False;
    short *sstyle;
    u_short newstyle, oldstyle;
 
+   if (areawin->selects == 0) {
+      preselected = FALSE;
+      if (value & BBOX)
+	 checkselect(POLYGON);
+      else
+	 checkselect(ARC | SPLINE | POLYGON | PATH);
+   }
+   else preselected = TRUE;
+
    if (areawin->selects > 0) {
       if (value & BBOX) {
 	 polyptr ckp;
@@ -1054,8 +1036,10 @@ int setelementstyle(xcWidget w, u_short value, u_short mask)
    }
 #ifndef TCL_WRAPPER
    setallstylemarks(newstyle);
-   unselect_all();
 #endif
+   if (!preselected)
+      unselect_all();
+
    return (int)newstyle;
 }
 
@@ -1077,8 +1061,10 @@ void setcolor(xcWidget w, int cindex)
 
    if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       labelptr curlabel = TOLABEL(EDITPART);
-      strptr = findstringpart(textpos - 1, NULL, curlabel->string, areawin->topinstance);
-      nextptr = findstringpart(textpos, NULL, curlabel->string, areawin->topinstance);
+      strptr = findstringpart(areawin->textpos - 1, NULL, curlabel->string,
+			areawin->topinstance);
+      nextptr = findstringpart(areawin->textpos, NULL, curlabel->string,
+			areawin->topinstance);
       if (strptr->type == FONT_COLOR) {
 	 undrawtext(curlabel);
 	 strptr->data.color = cindex;
@@ -1112,6 +1098,16 @@ void setcolor(xcWidget w, int cindex)
    }
 }
 
+/*--------------------------------------------------------------*/
+/* Set the menu checkmarks on the font menu			*/
+/*--------------------------------------------------------------*/
+
+void togglefontmark(int fontval)
+{
+   if (fonts[fontval].family != NULL)
+      XcInternalTagCall(xcinterp, 3, "label", "family", fonts[fontval].family);
+}
+
 /*------------------------------------------------------*/
 /* Set checkmarks on label style menu 			*/
 /* fvalue is for font, jvalue is for justification	*/
@@ -1119,106 +1115,51 @@ void setcolor(xcWidget w, int cindex)
 
 void togglestylemark(int styleval)
 {
-   switch(styleval) {
-      case 0: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontstyle",
-		"normal", 0);
-	 break;
-      case 1: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontstyle",
-		"bold", 0);
-	 break;
-      case 2: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontstyle",
-		"italic", 0);
-	 break;
-      case 3: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontstyle",
-		"bolditalic", 0);
-	 break;
-   }
+   char *cstyle = translatestyle(styleval);
+   if (cstyle != NULL)
+      XcInternalTagCall(xcinterp, 3, "label", "style", cstyle);
 }
 
+/*------------------------------------------------------*/
+/* Set checkmarks on label encoding menu		*/
 /*------------------------------------------------------*/
 
 void toggleencodingmark(int encodingval)
 {
-   switch(encodingval) {
-      case 0: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"Standard", 0);
-	 break;
-      case 1: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"special", 0);
-	 break;
-      case 2: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"ISOLatin1", 0);
-	 break;
-      case 3: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"ISOLatin2", 0);
-	 break;
-      case 4: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"ISOLatin3", 0);
-	 break;
-      case 5: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"ISOLatin4", 0);
-	 break;
-      case 6: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"ISOLatin5", 0);
-	 break;
-      case 7: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"ISOLatin6", 0);
-	 break;
-   }
+   char *cenc = translateencoding(encodingval);
+   if (cenc != NULL)
+      XcInternalTagCall(xcinterp, 3, "label", "encoding", cenc);
 }
 
+/*------------------------------------------------------*/
+/* Set checkmarks on label justification & flags menu	*/
 /*------------------------------------------------------*/
 
 void togglejustmarks(int justvalue)
 {
-   switch(justvalue & RLJUSTFIELD) {
-      case NORMAL: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),jhoriz",
-		"left", 0);
-	 break;
-      case NOTLEFT: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),jhoriz",
-		"center", 0);
-	 break;
-      case RIGHT|NOTLEFT: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),jhoriz",
-		"right", 0);
-	 break;
-   }
-   switch(justvalue & TBJUSTFIELD) {
-      case NORMAL: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),jvert",
-		"bottom", 0);
-	 break;
-      case NOTBOTTOM: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),jvert",
-		"middle", 0);
-	 break;
-      case TOP|NOTBOTTOM: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),jvert",
-		"top", 0);
-	 break;
-   }
-
-   /* Flip Invariance property */
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),flipinvariant",
-		(justvalue & FLIPINV) ?  "true" : "false", 0);
-
-   /* LaTeX mode property */
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),latexmode",
-		(justvalue & LATEXLABEL) ?  "true" : "false", 0);
-     
-   /* Pin visibility property */
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),pinvisible",
-		(justvalue & PINVISIBLE) ?  "true" : "false", 0);
+   XcInternalTagCall(xcinterp, 4, "label", "justify", (justvalue & RIGHT) ? "right" :
+	(justvalue & NOTLEFT) ? "center" : "left",
+	(justvalue & TOP) ? "top" : (justvalue & NOTBOTTOM) ? "middle" : "bottom");
+   XcInternalTagCall(xcinterp, 3, "label", "flipinvariant", (justvalue & FLIPINV) ?
+	"true" : "false");
+   XcInternalTagCall(xcinterp, 3, "label", "latex", (justvalue & LATEXLABEL) ?
+	"true" : "false");
+   XcInternalTagCall(xcinterp, 3, "label", "visible", (justvalue & PINVISIBLE) ?
+	"true" : "false");
 }
 
-/*------------------------------------------------------*/
+/*-------------------------------------------------------------*/
+/* Simultaneously set all relevant checkmarks for a text label */
+/*-------------------------------------------------------------*/
 
 void setfontmarks(short fvalue, short jvalue)
 {
-   if ((fvalue >= 0) && (fontcount > 0)) {
-      togglestylemark(fonts[fvalue].flags & 0x03);
-      toggleencodingmark((fonts[fvalue].flags & 0xf80) >> 7);
+   if ((fvalue >= 0) && (fvalue < fontcount)) {
+      toggleencodingmark(fvalue);
+      togglestylemark(fvalue);
       togglefontmark(fvalue);
    }
-   if (jvalue >= 0) {
-      togglejustmarks(jvalue);
-   }
+   togglejustmarks(jvalue);
 }
 
 #endif /* TCL_WRAPPER */
@@ -1277,261 +1218,6 @@ void setdefaultfontmarks()
    setfontmarks(areawin->psfont, areawin->justify);
 }
 
-/*--------------------------------------------------------------*/
-/* Toggle a bit in the justification field of a label		*/
-/*--------------------------------------------------------------*/
-
-void dojustifybit(xcWidget w, labelptr settext, short bitfield)
-{
-   if (settext != NULL) {
-      int oldjust = (int)settext->justify;
-
-      undrawtext(settext);
-      settext->justify ^= bitfield;
-      redrawtext(settext);
-      pwriteback(areawin->topinstance);
-
-      register_for_undo(XCF_Justify, UNDO_MORE, areawin->topinstance,
-		(genericptr)settext, oldjust);
-   }
-   else
-      areawin->justify ^= bitfield;
-
-#ifndef TCL_WRAPPER
-   if (w != NULL) {
-      Boolean boolval;
-      if (settext)
-	 boolval = (settext->justify & bitfield) ? 0 : 1;
-      else
-	 boolval = (areawin->justify & bitfield) ? 0 : 1;
-      toggle(w, (pointertype)(-1), &boolval);
-   }
-#endif
-}
-
-/*--------------------------------------------------------------*/
-/* Toggle a pin-related bit in the justification field of a	*/
-/* label.  This differs from dojustifybit() in that the label	*/
-/* must be a pin, and this function cannot change the default	*/
-/* behavior set by areawin->justify.				*/
-/*--------------------------------------------------------------*/
-
-void dopinjustbit(xcWidget w, labelptr settext, short bitfield)
-{
-   if ((settext != NULL) && settext->pin) {
-      int oldjust = (int)settext->justify;
-
-      undrawtext(settext);
-      settext->justify ^= bitfield;
-      redrawtext(settext);
-      pwriteback(areawin->topinstance);
-
-      register_for_undo(XCF_Justify, UNDO_MORE, areawin->topinstance,
-		(genericptr)settext, oldjust);
-
-#ifndef TCL_WRAPPER
-      if (w != NULL) {
-	 Boolean boolval = (settext->justify & bitfield) ? 0 : 1;
-	 toggle(w, (pointertype)(-1), &boolval);
-      }
-#endif
-   }
-}
-
-/*----------------------------------------------------------------*/
-/* Set the justification for the label passed as 3rd parameter	  */
-/*----------------------------------------------------------------*/
-
-void setjust(xcWidget w, pointertype value, labelptr settext, short mode)
-{
-   short newjust, oldjust;
-
-   if (settext != NULL) {
-
-      if (mode == 1)
-         newjust = (settext->justify & (NONJUSTFIELD | TBJUSTFIELD))
-			| value;
-      else
-         newjust = (settext->justify & (NONJUSTFIELD | RLJUSTFIELD))
-			| value;
-
-      if (settext->justify != newjust) {
-	  oldjust = (int)settext->justify;
-          undrawtext(settext);
-          settext->justify = newjust;
-          redrawtext(settext);
-          pwriteback(areawin->topinstance);
-
-          register_for_undo(XCF_Justify, UNDO_MORE, areawin->topinstance,
-		(genericptr)settext, oldjust);
-      }
-   }
-   else {
-      if (mode == 1)
-         newjust = (areawin->justify & (NONJUSTFIELD | TBJUSTFIELD))
-			| value;
-      else
-         newjust = (areawin->justify & (NONJUSTFIELD | RLJUSTFIELD))
-			| value;
-      areawin->justify = newjust;
-   }
-#ifndef TCL_WRAPPER
-   if (w != NULL) togglejustifs(w);
-#endif
-}
-
-/*----------------------------------------------------------------*/
-/* Set vertical justification (top, middle, bottom) on all	  */
-/* selected labels						  */
-/*----------------------------------------------------------------*/
-
-void setvjust(xcWidget w, pointertype value, caddr_t nulldata)
-{
-   short *fselect;
-   labelptr settext;
-   short labelcount = 0;
-
-   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
-      settext = *((labelptr *)EDITPART);
-      setjust(w, value, settext, 2);
-   }
-   else {
-      for (fselect = areawin->selectlist; fselect < areawin->selectlist +
-            areawin->selects; fselect++) {
-         if (SELECTTYPE(fselect) == LABEL) {
-            labelcount++;
-            settext = SELTOLABEL(fselect);
-            setjust(NULL, value, settext, 2);
-         }
-      }
-      if (labelcount == 0) setjust(w, value, NULL, 2);
-      else unselect_all();
-   }
-}
-
-/*----------------------------------------------------------------*/
-/* Set horizontal justification (left, center, right) on all	  */
-/* selected labels						  */
-/*----------------------------------------------------------------*/
-
-void sethjust(xcWidget w, pointertype value, caddr_t nulldata)
-{
-   short *fselect;
-   labelptr settext;
-   short labelcount = 0;
-
-   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
-      settext = *((labelptr *)EDITPART);
-      setjust(w, value, settext, 1);
-   }
-   else {
-      for (fselect = areawin->selectlist; fselect < areawin->selectlist +
-            areawin->selects; fselect++) {
-         if (SELECTTYPE(fselect) == LABEL) {
-            labelcount++;
-            settext = SELTOLABEL(fselect);
-            setjust(NULL, value, settext, 1);
-         }
-      }
-      if (labelcount == 0) setjust(w, value, NULL, 1);
-      else unselect_all();
-   }
-}
-
-/*--------------------------------------------------------------*/
-/* Set a justify field bit on all selected labels		*/ 
-/* (flip invariance, latex mode, etc.)				*/
-/*--------------------------------------------------------------*/
-
-void setjustbit(xcWidget w, pointertype value, caddr_t nulldata)
-{
-   short *fselect;
-   labelptr settext;
-   short labelcount = 0;
-
-   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
-      settext = *((labelptr *)EDITPART);
-      dojustifybit(w, settext, (short)value);
-   }
-   else {
-      for (fselect = areawin->selectlist; fselect < areawin->selectlist +
-            areawin->selects; fselect++) {
-         if (SELECTTYPE(fselect) == LABEL) {
-            labelcount++;
-            settext = SELTOLABEL(fselect);
-            dojustifybit(NULL, settext, (short)value);
-         }
-      }
-      if (labelcount == 0) dojustifybit(w, NULL, (short)value);
-      else unselect_all();
-   }
-}
-
-/*--------------------------------------------------------------*/
-/* Set pin-related bit field of "justify" on all selected pins	*/ 
-/* (e.g., pin visibility)					*/
-/*--------------------------------------------------------------*/
-
-void setpinjustbit(xcWidget w, pointertype value, caddr_t nulldata)
-{
-   short *fselect;
-   labelptr settext;
-
-   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
-      settext = *((labelptr *)EDITPART);
-      if (settext->pin)
-	 dopinjustbit(w, settext, (short)value);
-   }
-   else {
-      for (fselect = areawin->selectlist; fselect < areawin->selectlist +
-            areawin->selects; fselect++) {
-         if (SELECTTYPE(fselect) == LABEL) {
-            settext = SELTOLABEL(fselect);
-	    if (settext->pin)
-               dopinjustbit(NULL, settext, (short)value);
-         }
-      }
-      unselect_all();
-   }
-}
-
-/*---------------------------*/
-/* Set polygon editing style */
-/*---------------------------*/
-
-#ifdef TCL_WRAPPER
-
-void boxedit(xcWidget w, pointertype value, caddr_t nulldata)
-{
-   switch (value) {
-      case MANHATTAN:
-	 Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),polyedittype",
-		"manhattan", 0);
-	 break;
-      case RHOMBOIDX:
-	 Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),polyedittype",
-		"rhomboidx", 0);
-	 break;
-      case RHOMBOIDY:
-	 Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),polyedittype",
-		"rhomboidy", 0);
-	 break;
-      case RHOMBOIDA:
-	 Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),polyedittype",
-		"rhomboida", 0);
-	 break;
-      case NORMAL:
-	 Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),polyedittype",
-		"normal", 0);
-	 break;
-   }
-
-   if (areawin->boxedit == value) return;
-   areawin->boxedit = value;
-}
-
-#endif /* TCL_WRAPPER */
-
 /*----------------------------------------------------------------*/
 /* Pick up font name from _STR2 and pass it to loadfontfile()	  */
 /*----------------------------------------------------------------*/
@@ -1657,9 +1343,9 @@ void setfontval(xcWidget w, pointertype value, labelptr settext)
 
       /* if last byte was a font designator, use it */
 
-      if (textpos > 0 || textpos < stringlength(settext->string, True,
-			areawin->topinstance)) {
-	 strptr = findstringpart(textpos - 1, NULL, settext->string,
+      if (areawin->textpos > 0 || areawin->textpos < stringlength(settext->string,
+			True, areawin->topinstance)) {
+	 strptr = findstringpart(areawin->textpos - 1, NULL, settext->string,
 			areawin->topinstance);
 	 if (strptr->type == FONT_NAME) {
 	    tc = strptr->data.font;
@@ -1678,7 +1364,7 @@ void setfontval(xcWidget w, pointertype value, labelptr settext)
       }
 
       /* otherwise, look for the last style used in the string */
-      tc = findcurfont(textpos, settext->string, areawin->topinstance);
+      tc = findcurfont(areawin->textpos, settext->string, areawin->topinstance);
    }
    else tc = areawin->psfont;
 
@@ -1709,6 +1395,7 @@ void setfont(xcWidget w, pointertype value, caddr_t calldata)
    short *fselect;
    labelptr settext;
    short labelcount = 0;
+   Boolean preselected;
 
    if (eventmode == CATALOG_MODE || eventmode == FONTCAT_MODE ||
 		eventmode == EFONTCAT_MODE) return;
@@ -1719,7 +1406,12 @@ void setfont(xcWidget w, pointertype value, caddr_t calldata)
       charreport(settext);
    }
    else {
-      textpos = 1;
+      if (areawin->selects == 0) {
+	 checkselect(LABEL);
+	 preselected = FALSE;
+      }
+      else preselected = TRUE;
+      areawin->textpos = 1;
       for (fselect = areawin->selectlist; fselect < areawin->selectlist +
 	    areawin->selects; fselect++) {
          if (SELECTTYPE(fselect) == LABEL) {
@@ -1729,7 +1421,7 @@ void setfont(xcWidget w, pointertype value, caddr_t calldata)
          }
       }
       if (labelcount == 0) setfontval(w, value, NULL);
-      else unselect_all();
+      else if (!preselected) unselect_all();
    }
 }
 
@@ -1748,8 +1440,10 @@ void setfontstyle(xcWidget w, pointertype value, labelptr settext)
 
       /* if last byte was a font designator, use it */
 
-      if (textpos > 0 || textpos < stringlength(settext->string, True, areawin->topinstance)) {
-	 strptr = findstringpart(textpos - 1, NULL, settext->string, areawin->topinstance);
+      if (areawin->textpos > 0 || areawin->textpos < stringlength(settext->string,
+			True, areawin->topinstance)) {
+	 strptr = findstringpart(areawin->textpos - 1, NULL, settext->string,
+			areawin->topinstance);
 	 if (strptr->type == FONT_NAME) {
 	    tc = strptr->data.font;
 
@@ -1771,8 +1465,11 @@ void setfontstyle(xcWidget w, pointertype value, labelptr settext)
          }
       }
 
-      /* otherwise, look for the last font used in the string */
-      tc = findcurfont(textpos - 2, settext->string, areawin->topinstance);
+      /* Otherwise, look for the last font used in the string */
+      /* Fix by Dimitri Princen.  Was areawin->textpos - 2; was there a */
+      /* reason for that?						*/
+
+      tc = findcurfont(areawin->textpos, settext->string, areawin->topinstance);
    }
    else tc = areawin->psfont;
 
@@ -1803,6 +1500,7 @@ void fontstyle(xcWidget w, pointertype value, caddr_t nulldata)
    short *fselect;
    labelptr settext;
    short labelcount = 0;
+   Boolean preselected;
 
    if (eventmode == CATALOG_MODE || eventmode == FONTCAT_MODE ||
 		eventmode == EFONTCAT_MODE) return;
@@ -1813,7 +1511,12 @@ void fontstyle(xcWidget w, pointertype value, caddr_t nulldata)
       charreport(settext);
    }
    else {
-      textpos = 1;
+      if (areawin->selects == 0) {
+	 checkselect(LABEL);
+	 preselected = FALSE;
+      }
+      else preselected = TRUE;
+      areawin->textpos = 1;
       for (fselect = areawin->selectlist; fselect < areawin->selectlist +
 	    areawin->selects; fselect++) {
          if (SELECTTYPE(fselect) == LABEL) {
@@ -1823,7 +1526,7 @@ void fontstyle(xcWidget w, pointertype value, caddr_t nulldata)
          }
       }
       if (labelcount == 0) setfontstyle(w, value, NULL);
-      else unselect_all();
+      else if (!preselected) unselect_all();
    }
 }
 
@@ -1842,9 +1545,9 @@ void setfontencoding(xcWidget w, pointertype value, labelptr settext)
 
       /* if last byte was a font designator, use it */
 
-      if (textpos > 0 || textpos < stringlength(settext->string, True,
-			areawin->topinstance)) {
-	 strptr = findstringpart(textpos - 1, NULL, settext->string,
+      if (areawin->textpos > 0 || areawin->textpos < stringlength(settext->string,
+		True, areawin->topinstance)) {
+	 strptr = findstringpart(areawin->textpos - 1, NULL, settext->string,
 			areawin->topinstance);
 	 if (strptr->type == FONT_NAME) {
 	    tc = strptr->data.font;
@@ -1868,7 +1571,7 @@ void setfontencoding(xcWidget w, pointertype value, labelptr settext)
       }
 
       /* otherwise, look for the last style used in the string */
-      tc = findcurfont(textpos - 2, settext->string, areawin->topinstance);
+      tc = findcurfont(areawin->textpos - 2, settext->string, areawin->topinstance);
    }
    else tc = areawin->psfont;
 
@@ -1898,6 +1601,7 @@ void fontencoding(xcWidget w, pointertype value, caddr_t nulldata)
    short *fselect;
    labelptr settext;
    short labelcount = 0;
+   Boolean preselected;
 
    if (eventmode == CATALOG_MODE || eventmode == FONTCAT_MODE ||
 		eventmode == EFONTCAT_MODE) return;
@@ -1908,7 +1612,12 @@ void fontencoding(xcWidget w, pointertype value, caddr_t nulldata)
       charreport(settext);
    }
    else {
-      textpos = 1;
+      if (areawin->selects == 0) {
+  	 checkselect(LABEL);
+	 preselected = FALSE; 
+      }
+      else preselected = TRUE;
+      areawin->textpos = 1;
       for (fselect = areawin->selectlist; fselect < areawin->selectlist +
 	    areawin->selects; fselect++) {
          if (SELECTTYPE(fselect) == LABEL) {
@@ -1918,7 +1627,7 @@ void fontencoding(xcWidget w, pointertype value, caddr_t nulldata)
          }
       }
       if (labelcount == 0) setfontencoding(w, value, NULL);
-      else unselect_all();
+      else if (!preselected) unselect_all();
    }
 }
 
@@ -1939,7 +1648,7 @@ Boolean dospecial()
    curlabel = TOLABEL(EDITPART);
    if (curlabel->justify & LATEXLABEL) return False;
 
-   cfont = findcurfont(textpos, curlabel->string, areawin->topinstance);
+   cfont = findcurfont(areawin->textpos, curlabel->string, areawin->topinstance);
    composefontlib(cfont);
    startcatalog(NULL, FONTLIB, NULL);
    return True;
