@@ -7,6 +7,7 @@
 
 /* Note variable argument list for register_for_undo() */
 extern void register_for_undo(u_int, u_char, objinstptr, ...);
+extern void undo_finish_series(void);
 extern void undo_action(void);
 extern void redo_action(void);
 extern void flush_undo_stack(void);
@@ -38,6 +39,11 @@ extern Tcl_Obj *evaluate_raw(objectptr, oparamptr, objinstptr, int *);
 extern char *TCL_to_PS(char *);
 extern XCWindowData *GUI_init(int, Tcl_Obj *CONST objv[]);
 extern void build_app_database(Tk_Window);
+extern int XcInternalTagCall(Tcl_Interp *, int, ...);
+extern char *translateparamtype(int);
+extern char *translatestyle(int);
+extern char *translateencoding(int);
+
 #endif
 
 extern char *evaluate_expr(objectptr, oparamptr, objinstptr);
@@ -61,6 +67,7 @@ extern void FillPolygon(Display *, Window, GC, XPoint *, int, int, int);
 /* from elements.c: */
 
 /* element constructor functions */
+extern int getsubpartindex(pathptr);
 extern labelptr new_label(objinstptr, stringpart *, int, int, int);
 extern labelptr new_simple_label(objinstptr, char *, int, int, int);
 extern labelptr new_temporary_label(objectptr, char *, int, int);
@@ -213,7 +220,7 @@ extern void polycopy(polyptr, polyptr);
 extern void splinecopy(splineptr, splineptr);
 extern void pathcopy(pathptr, pathptr);
 extern void instcopy(objinstptr, objinstptr);
-extern void delete_tagged(objectptr);
+extern void delete_tagged(objinstptr);
 extern void createcopies(void);
 extern void copydrag(void);
 extern void copy_op(int, int, int);
@@ -221,6 +228,7 @@ extern Boolean checkmultiple(XButtonEvent *);
 extern void continue_op(int, int, int);
 extern void finish_op(int, int, int);
 extern void path_op(genericptr, int, int, int);
+extern void inst_op(genericptr, int, int, int);
 extern void resizearea(xcWidget, caddr_t, caddr_t);
 extern void drawarea(xcWidget, caddr_t, caddr_t);
 extern void standard_element_delete(short);
@@ -267,7 +275,9 @@ extern void varcheck(FILE *, short, objectptr, short *, genericptr, u_char);
 extern void varfcheck(FILE *, float, objectptr, short *, genericptr, u_char);
 extern void varpcheck(FILE *, short, objectptr, int, short *, genericptr, u_char);
 extern void getfile(xcWidget, pointertype, caddr_t);
+extern int filecmp(char *, char *);
 extern Boolean nextfilename(void);
+extern FILE *libopen(char *, short, char *, int);
 extern void loadfontlib(void);
 extern void loadglib(Boolean, short, short);
 extern void loadulib(void);
@@ -301,6 +311,10 @@ extern TechPtr LookupTechnology(char *);
 extern TechPtr GetObjectTechnology(objectptr);
 extern TechPtr AddNewTechnology(char *, char *);
 extern void AddObjectTechnology(objectptr);
+extern void TechReplaceSave();
+extern void TechReplaceRestore();
+extern void TechReplaceAll();
+extern void TechReplaceNone();
 
 #ifdef TCL_WRAPPER
 extern void setfile(char *, int);
@@ -394,7 +408,7 @@ extern void InvTransformPoints(XPoint *, XPoint *, short, XPoint, float,
 extern void manhattanize(XPoint *, polyptr, short, Boolean);
 extern void bboxcalc(short, short *, short *);
 extern void calcextents(genericptr *, short *, short *, short *, short *);
-extern void objinstbbox(objinstptr, XPoint *, Boolean);
+extern void objinstbbox(objinstptr, XPoint *, int);
 extern void labelbbox(labelptr, XPoint *, objinstptr);
 extern void graphicbbox(graphicptr, XPoint *);
 extern void calcinstbbox(genericptr *, short *, short *, short *, short *);
@@ -424,7 +438,7 @@ extern void UDrawTextLine(labelptr, short);
 extern void UDrawTLine(labelptr);
 extern void UDrawXLine(XPoint, XPoint);
 extern void UDrawBox(XPoint, XPoint);
-extern void UDrawRescaleBox(XPoint *);
+extern float UDrawRescaleBox(XPoint *);
 extern void UDrawBBox(void);
 extern void strokepath(XPoint *, short, short, float);
 extern void makesplinepath(splineptr, XPoint *);
@@ -629,9 +643,10 @@ extern void addnewfont(xcWidget, caddr_t, caddr_t);
 #ifdef TCL_WRAPPER
 extern Tcl_Obj *tclglobals(objinstptr);
 extern Tcl_Obj *tcltoplevel(objinstptr);
-void ratsnest(objectptr);
+void ratsnest(objinstptr);
 #endif
 
+extern void ReferencePosition(objinstptr, XPoint *, XPoint *);
 extern int NameToPinLocation(objinstptr, char *, int *, int *);
 extern Boolean RemoveFromNetlist(objectptr, genericptr);
 extern labelptr NetToLabel(int, objectptr);
@@ -749,9 +764,13 @@ extern genericptr findparam(objectptr, void *, u_char);
 extern Boolean paramcross(objectptr, labelptr);
 extern oparamptr parampos(objectptr, labelptr, char *, short *, short *);
 extern int opsubstitute(objectptr, objinstptr);
+extern void exprsub(genericptr);
+extern int epsubstitute(genericptr, objectptr, objinstptr, Boolean *);
 extern int psubstitute(objinstptr);
 extern Boolean has_param(genericptr);
 extern oparamptr copyparameter(oparamptr);
+extern eparamptr copyeparam(eparamptr, genericptr);
+
 extern void pwriteback(objinstptr);
 extern short paramlen(u_char *);
 extern int natstrlen(u_char *);
@@ -841,12 +860,12 @@ extern void dopintype(xcWidget, pointertype, caddr_t);
 extern void setsymschem(void);
 extern int findpageobj(objectptr);
 extern void collectsubschems(int);
-extern int findsubschems(int, objectptr, int, short *);
+extern int findsubschems(int, objectptr, int, short *, Boolean);
 extern void copypinlabel(labelptr);
 extern int checkschem(objectptr, char *);
 extern int checksym(objectptr, char *);
-extern void changeotherpins(labelptr, stringpart *);
-extern void swapschem(int, int);
+extern int changeotherpins(labelptr, stringpart *);
+extern void swapschem(int, int, char *);
 extern void dobeforeswap(xcWidget, caddr_t, caddr_t);
 extern void schemdisassoc(void);
 extern void startschemassoc(xcWidget, pointertype, caddr_t);
