@@ -35,7 +35,6 @@
 extern Display *dpy;
 extern XCWindowData *areawin;
 extern Globaldata xobjs;
-extern short textpos, textend;
 extern short fontcount;
 extern fontinfo *fonts;
 extern int *appcolors;
@@ -200,7 +199,7 @@ stringpart *splitstring(int tpos, stringpart **strtop, objinstptr localinst)
 }
 
 /*----------------------------------------------------------------------*/
-/* Get the next string part, linking to a paramter if necessary		*/
+/* Get the next string part, linking to a parameter if necessary	*/
 /*----------------------------------------------------------------------*/
 
 stringpart *nextstringpartrecompute(stringpart *strptr, objinstptr thisinst)
@@ -373,7 +372,8 @@ stringpart *linkstring(objinstptr localinst, stringpart *strstart,
 
    if (localinst == NULL) {
       ops = match_param(topobject, key);
-      if (ops == NULL) return NULL;
+      if (ops == NULL)
+	 return NULL;
    }
    else {
       ops = find_param(localinst, key);
@@ -458,7 +458,8 @@ int findcurfont(int tpos, stringpart *strtop, objinstptr thisinst)
    stringpart *strptr;
 
    curpos = findstringpart(tpos, NULL, strtop, thisinst);
-   for (strptr = strtop; strptr != curpos; strptr = nextstringpart(strptr, thisinst))
+   for (strptr = strtop; (strptr != NULL) && (strptr != curpos);
+		strptr = nextstringpart(strptr, thisinst))
       if (strptr->type == FONT_NAME)
          cfont = strptr->data.font;
 
@@ -561,7 +562,8 @@ void charprint(char *sout, stringpart *strptr, int locpos)
 	    *sout = '\0';
 	 break;
       case FONT_NAME:
-	 sprintf(sout, "Font=%s", fonts[strptr->data.font].psname);
+	 sprintf(sout, "Font=%s", (strptr->data.font >= fontcount) ?
+		"(unknown)" : fonts[strptr->data.font].psname);
 	 break;
       case FONT_SCALE:
 	 sprintf(sout, "Scale=%3.2f", strptr->data.scale);
@@ -1321,6 +1323,9 @@ short UDrawChar(u_char code, short styles, short ffont, int groupheight, int pas
    short  localwidth;
    objinst charinst;
 
+   if ((ffont >= fontcount) || (fonts[ffont].encoding == NULL))
+      return 0;
+
    alphapts[0].x = 0;
    alphapts[0].y = 0;
    charinst.type = OBJINST;
@@ -1453,12 +1458,14 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
 
 	  switch(strptr->type) {
 	     case FONT_NAME:
-		ffont = strptr->data.font;
-		fstyle = 0;		   /* style reset by font change */
-	        if (baseline == newpoint.y) {  /* set top-level font and style */
-	           oldfont = ffont;
-	           oldstyle = fstyle;
-	        }
+		if (strptr->data.font < fontcount) {
+		   ffont = strptr->data.font;
+		   fstyle = 0;		   /* style reset by font change */
+	           if (baseline == newpoint.y) {  /* set top-level font and style */
+	              oldfont = ffont;
+	              oldstyle = fstyle;
+	           }
+		}
 		
 		/* simple boldface technique for derived fonts */
 
@@ -1587,7 +1594,7 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
 		   char *nsptr = strstr(textptr, "::");
 		   if (nsptr != NULL) {
 		      textptr = nsptr + 2;
-		      pos += (int)nsptr - (int)strptr->data.string + 2;
+		      pos += (pointertype)nsptr - (pointertype)strptr->data.string + 2;
 		   }
 		}
 
@@ -1601,9 +1608,9 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
 
 		   scolor = curcolor;
 	           if (passcolor == DOSUBSTRING) {
-	              if (pos <= textpos && pos > textend)
+	              if (pos <= areawin->textpos && pos > areawin->textend)
 		         scolor = SELECTCOLOR;
-		      else if (pos > textpos) {
+		      else if (pos > areawin->textpos) {
       		         XTopSetForeground(curcolor);
 		      }
 	           }
@@ -1758,7 +1765,7 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
 		char *nsptr = strstr(textptr, "::");
 		if (nsptr != NULL) {
 		   textptr = nsptr + 2;
-		   locpos += (int)nsptr - (int)strptr->data.string + 2;
+		   locpos += (pointertype)nsptr - (pointertype)strptr->data.string + 2;
 		}
 	    }
 
