@@ -85,6 +85,8 @@ typedef struct {
    float scale;			/* old scale value */
 } scaleinfo;
 
+Boolean undo_collect = FALSE;
+
 /*----------------------------------------------------------------------*/
 /* Externally declared variables					*/
 /*----------------------------------------------------------------------*/
@@ -144,7 +146,7 @@ uselection *remember_selection(objinstptr topinst, short *slist, int number)
 }
 
 /*----------------------------------------------------------------------*/
-/* Create a selection list in areawin from the saved uselection	*/
+/* Create a selection list in areawin from the saved uselection		*/
 /* record.								*/
 /*----------------------------------------------------------------------*/
 
@@ -372,7 +374,9 @@ void undelete_one_element(objinstptr thisinst, genericptr thiselem)
 /*									*/
 /*   "mode" is UNDO_MORE if one or more undo records are expected to 	*/
 /*   follow in a series, or UNDO_DONE if this completes a series, or is	*/
-/*   as single undo event.						*/
+/*   as single undo event. The command-line command "undo series start"	*/
+/*   forces all events to be UNDO_MORE until "undo series end" is	*/
+/*   issued.								*/
 /*----------------------------------------------------------------------*/
 
 void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
@@ -386,7 +390,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
    uselection *srec;
    editelement *erec;
    scaleinfo *escale;
-   genericptr egen, *ggen, *cgen;
+   genericptr egen;
    XPoint *fpoint;
    double scale;
 
@@ -418,7 +422,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
    else
       newrecord->idx = 1;
 
-   if (mode == UNDO_MORE)
+   if (mode == UNDO_MORE || undo_collect == TRUE)
       newrecord->idx = -newrecord->idx;
    
    xobjs.undostack = newrecord;
@@ -555,7 +559,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 	 newrecord->idata = oval;
 	 break;
 
-      case XCF_Scale:
+      case XCF_Rescale:
 	 /* 2 args:							*/
 	 /*	egen = element that was changed (with new value)	*/
 	 /*	ofloat = old float value				*/
@@ -812,7 +816,6 @@ short undo_one_action()
 	       int tmpnum;
 	       polyptr epoly;
 	       splineptr espline;
-	       pathinfo *tmpinfo = erec->save.pathspecs;
 	       pathptr epath = (pathptr)(erec->element);
 	       for (i = 0; i < epath->parts; i++) {
 		  genericptr ggen = *(epath->plist + i);
@@ -893,7 +896,7 @@ short undo_one_action()
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
-      case XCF_Scale:
+      case XCF_Rescale:
 	 escale = (scaleinfo *)thisrecord->undodata;
 	 egen = escale->element;
 	 fnum = escale->scale;
@@ -929,6 +932,11 @@ short undo_one_action()
 	       thisgraphic->scale = fnum;
 	       thisgraphic->valid = FALSE;
 	       break;
+	    case LABEL:
+	       thislabel = (labelptr)egen;
+	       escale->scale = thislabel->scale;
+	       thislabel->scale = fnum;
+	       break;
 	 }
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
@@ -981,6 +989,21 @@ short undo_one_action()
    return thisrecord->idx;
 }
 
+/*----------------------------------------------------------------------*/
+/* undo_finish_series ---						*/
+/*	Complete a possibly incomplete undo series by forcing the	*/
+/*	topmost entry to have a positive index.				*/
+/*	Note that for "undo series start|end" to work, undo_collect	*/
+/*	must be set to FALSE prior to calling undo_finish_series().	*/
+/*----------------------------------------------------------------------*/
+
+void undo_finish_series()
+{
+   if (undo_collect == FALSE)
+      if (xobjs.undostack && xobjs.undostack->idx < 0)
+         xobjs.undostack->idx = -xobjs.undostack->idx;
+}
+
 /*----------------------------------------------------------------------*/
 /* undo_action ---							*/
 /*	Play undo record back to the completion of a series.		*/
@@ -1183,7 +1206,6 @@ short redo_one_action()
 	       int tmpnum;
 	       polyptr epoly;
 	       splineptr espline;
-	       pathinfo *tmpinfo = erec->save.pathspecs;
 	       pathptr epath = (pathptr)(erec->element);
 	       for (i = 0; i < epath->parts; i++) {
 		  genericptr ggen = *(epath->plist + i);
@@ -1252,7 +1274,7 @@ short redo_one_action()
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
-      case XCF_Scale:
+      case XCF_Rescale:
 	 escale = (scaleinfo *)thisrecord->undodata;
 	 egen = escale->element;
 	 fnum = escale->scale;
@@ -1288,6 +1310,11 @@ short redo_one_action()
 	       thisgraphic->scale = fnum;
 	       thisgraphic->valid = FALSE;
 	       break;
+	    case LABEL:
+	       thislabel = (labelptr)egen;
+	       escale->scale = thislabel->scale;
+	       thislabel->scale = fnum;
+	       break;
 	 }
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
