@@ -99,7 +99,7 @@
 char	 _STR2[250];  /* Specifically for text returned from the popup prompt */
 char	 _STR[150];   /* Generic multipurpose string */
 short	 popups;      /* total number of popup widgets on the screen */
-Boolean	 pressmode;   /* Whether we are in a press & hold state */
+int	 pressmode;   /* Whether we are in a press & hold state */
 Display  *dpy = NULL;        /* Works well to make this globally accessible */
 Colormap cmap;
 Pixmap   STIPPLE[STIPPLES];  /* Polygon fill-style stipple patterns */
@@ -120,7 +120,6 @@ fontinfo *fonts;
 /* Externally defined variables						*/
 /*----------------------------------------------------------------------*/
 
-extern short textpos;
 extern aliasptr aliastop;
 extern float version;
 
@@ -206,13 +205,15 @@ void quit(xcWidget w, caddr_t nulldata)
    Matrixptr curmatrix, dmatrix;
 
    /* free up CTM Matrix Stack */
-   curmatrix = areawin->MatStack;
-   while (curmatrix != NULL) {
-      dmatrix = curmatrix->nextmatrix;
-      free(curmatrix);
-      curmatrix = dmatrix;
+   if (areawin != NULL) {
+      curmatrix = areawin->MatStack;
+      while (curmatrix != NULL) {
+         dmatrix = curmatrix->nextmatrix;
+         free(curmatrix);
+         curmatrix = dmatrix;
+      }
+      areawin->MatStack = NULL;
    }
-   areawin->MatStack = NULL;
 
    /* free the colormap if a new one has been installed */
 
@@ -386,7 +387,7 @@ void quitcheck(xcWidget w, caddr_t clientdata, caddr_t calldata)
    }
    else {
       free(promptstr);
-      quit(areawin->area, NULL);
+      quit(w, NULL);
    }
 }
 
@@ -577,6 +578,9 @@ void printeventmode() {
       case EPATH_MODE:
 	 Fprintf(stderr, "EPATH");
 	 break;
+      case EINST_MODE:
+	 Fprintf(stderr, "EINST");
+	 break;
       case ASSOC_MODE:
 	 Fprintf(stderr, "ASSOC");
 	 break;
@@ -971,7 +975,7 @@ void delete_window(XCWindowData *window)
    XCWindowData *searchwin, *lastwin = NULL;
 
    if (xobjs.windowlist->next == NULL) {
-      quitcheck(NULL, NULL, NULL);
+      quitcheck((window == NULL) ? NULL : window->area, NULL, NULL);
       return;
    }
 
@@ -1134,7 +1138,6 @@ void pre_initialize()
    xobjs.showtech = FALSE;
    xobjs.suspend = (char)0;	/* Suspend graphics until finished with startup */
    xobjs.new_changes = 0;
-   xobjs.do_replace = -1;	/* default: replace NONE */
    xobjs.filefilter = TRUE;
    xobjs.tempfile = NULL;
    xobjs.retain_backup = False;	/* default: remove backup after file write */
@@ -1171,13 +1174,16 @@ void pre_initialize()
 
    fontcount = 0;
    fonts = (fontinfo *) malloc(sizeof(fontinfo));
+   fonts[0].encoding = NULL;	/* To prevent segfaults */
+   fonts[0].psname = NULL;
+   fonts[0].family = NULL;
 
    /* Initialization of objects requires values for the window width and height, */
    /* so set up the widgets and realize them first.				 */
 
    popups = 0;        /* no popup windows yet */
    beeper = 1;        /* Ring bell on certain warnings or errors */
-   pressmode = FALSE; /* not in a button press & hold mode yet */
+   pressmode = 0;	/* not in a button press & hold mode yet */
    initsplines();	/* create lookup table of spline parameters */
 }
 
