@@ -107,7 +107,7 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #define ARC		0x08
 #define SPLINE  	0x10
 #define PATH		0x20
-#define GRAPHIC		0x40	/* reserved; unused, for now.		*/
+#define GRAPHIC		0x40
 #define ARRAY		0x80	/* reserved; unused, for now.		*/
 
 #define REMOVE_TAG	0x100	/* element to be removed from netlist	*/
@@ -272,6 +272,8 @@ typedef struct {
 #define INTSEGS		(SPLINESEGS - 2)
 #define LASTSEG		(SPLINESEGS - 3)
 
+#define NOFILENAME	(char *)(-1)
+
 #define FONTHEIGHT(y)	 (y->ascent + y->descent + 6) 
 #define ROWHEIGHT	 FONTHEIGHT(appdata.xcfont)
 #define FILECHARASCENT   (appdata.filefont->ascent)
@@ -307,8 +309,8 @@ enum editmode {
   RESCALE_MODE,		/* Interactive element rescaling box */
   CATALOG_MODE,		/* On a library page, library directory, or page directory */
   CATTEXT_MODE,		/* Editing an existing object name in the library */
-  FONTCAT_MODE,		/* Accessing the font character page from TEXT2_MODE */
-  EFONTCAT_MODE,	/* Accessing the font character page from TEXT3_MODE */
+  FONTCAT_MODE,		/* Accessing the font character page from TEXT_MODE */
+  EFONTCAT_MODE,	/* Accessing the font character page from ETEXT_MODE */
   TEXT_MODE,		/* Creating a new label */
   WIRE_MODE,		/* Creating a new polygon (wire) */
   BOX_MODE,		/* Creating a new box */
@@ -319,6 +321,7 @@ enum editmode {
   EARC_MODE,		/* Editing an existing arc */
   ESPLINE_MODE,		/* Editing an existing spline */
   EPATH_MODE,		/* Editing an existing path */
+  EINST_MODE,		/* Editing an instance (from the level above) */
   ASSOC_MODE,		/* Choosing an associated schematic or symbol */
   CATMOVE_MODE		/* Moving objects in or between libraries */
 };
@@ -411,6 +414,7 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 /* Library types							*/
 /*----------------------------------------------------------------------*/
 
+#define FONTENCODING   -1  /* Used only by libopen() */
 #define FONTLIB		0
 #define PAGELIB		1
 #define LIBLIB		2
@@ -477,6 +481,7 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define SYMBOL 3		/* Symbol associated with a schematic */
 #define FUNDAMENTAL 4		/* Standalone symbol */
 #define NONETWORK 5		/* Do not netlist this object */
+#define GLYPH 6			/* Symbol is a font glyph */
 
 #define LOCAL 1
 #define GLOBAL 2
@@ -498,8 +503,9 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define INDEPENDENT	0
 #define DEPENDENT	1
 #define TOTAL_PAGES	2
-#define PAGE_DEPEND	3
-#define ALL_PAGES	4
+#define LINKED_PAGES	3
+#define PAGE_DEPEND	4
+#define ALL_PAGES	5
 
 /*----------------------------------------------------------------------*/
 /* Color scheme styles (other than NORMAL)				*/
@@ -538,22 +544,25 @@ typedef XfPoint* fpointlist;
 /* Allowed parameterization types					*/
 /*----------------------------------------------------------------------*/
 
-#define P_NUMERIC	0	/* uncommitted numeric parameter */
-#define P_SUBSTRING	1
-#define P_POSITION_X	2
-#define P_POSITION_Y	3
-#define P_STYLE		4
-#define P_JUSTIFY	5
-#define P_ANGLE1	6
-#define P_ANGLE2	7
-#define P_RADIUS	8
-#define P_MINOR_AXIS	9
-#define P_ROTATION	10
-#define P_SCALE		11
-#define P_LINEWIDTH	12
-#define P_COLOR		13
-#define P_EXPRESSION	14
-#define P_POSITION	15	/* mode only, not a real parameter */
+enum paramwhich {
+   P_NUMERIC = 0,	/* uncommitted numeric parameter */
+   P_SUBSTRING,
+   P_POSITION_X,
+   P_POSITION_Y,
+   P_STYLE,
+   P_JUSTIFY,
+   P_ANGLE1,
+   P_ANGLE2,
+   P_RADIUS,
+   P_MINOR_AXIS,
+   P_ROTATION,
+   P_SCALE,
+   P_LINEWIDTH,
+   P_COLOR,
+   P_EXPRESSION,
+   P_POSITION, 		/* mode only, not a real parameter */
+   NUM_PARAM_TYPES
+};
 
 /*----------------------------------------------------------------------*/
 /* Labels are constructed of strings and executables 			*/
@@ -611,13 +620,17 @@ typedef struct _eparam *eparamptr;
 
 typedef struct _eparam {
    char *	key;	    /* name of the parameter */
+   u_char	flags;	    /* namely, bit declaring an indirect parameter */
    union {
       int	pointno;    /* point number in point array, for polygons */
+      short	pathpt[2];  /* element number and point number, for paths */
       char	*refkey;    /* parameter reference key, for instances */
    } pdata;
    eparamptr	next;	    /* next parameter in linked list */
 } eparam;
 
+#define P_INDIRECT	0x01	/* indirect parameter indicator */
+
 /*----------------------------------------------------------------------*/
 /* Generic element type	is a superset of all elements.			*/	
 /*----------------------------------------------------------------------*/
@@ -710,6 +723,8 @@ typedef struct {
    float	  scale;
    XImage	  *source;	/* source data */
    XImage	  *target;	/* target (scaled) data */
+   short	  trot;		/* target rotation */
+   float	  tscale;	/* target scale (0 = uninitialized) */
    Pixmap	  clipmask;	/* clipmask for non-manhattan rotations */
    Boolean	  valid;	/* does target need to be regenerated? */
 } graphic, *graphicptr;
@@ -722,6 +737,7 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
+   short	cycle;		/* Edit position, or -1	*/
    XPoint	position;
    short	rotation;
    float	scale;
@@ -738,6 +754,7 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
+   short	cycle;		/* Edit position, or -1	*/
    u_short	style;
    float	width;
    short	number;
@@ -752,6 +769,7 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
+   short	cycle;		/* Edit position, or -1	*/
    u_short	style;
    float	width;
    XPoint	ctrl[4];
@@ -767,11 +785,12 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
+   short	cycle;		/* Edit position, or -1	*/
    u_short	style;
    float	width;
    short	radius; 	/* x-axis radius */
    short	yaxis;		/* y-axis radius */
-   float	angle1;
+   float	angle1;		/* endpoint angles, in degrees */
    float	angle2;
    XPoint	position;
    /* the following are for rendering only */
@@ -990,7 +1009,7 @@ typedef struct _matrix {
 /* button tap/press definitions 					*/
 /*----------------------------------------------------------------------*/
 
-#define PRESSTIME	150	/* milliseconds of push to be a "press" */
+#define PRESSTIME	200	/* milliseconds of push to be a "press" */
 
 /*----------------------------------------------------------------------*/
 /* object name alias structures						*/
@@ -1053,7 +1072,6 @@ typedef struct _alias {
 #ifndef ClientData
    #define ClientData			XtPointer
 #endif
-
 #endif
 
 /*----------------------------------------------------------------------*/
@@ -1218,7 +1236,7 @@ enum {
    XCF_Text_Split /* 24 */,	XCF_Text_Home /* 25 */,
    XCF_Text_End /* 26 */,	XCF_Linebreak /* 27 */,
    XCF_Parameter /* 28 */,	XCF_Edit_Param /* 29 */,
-   XCF_Edit_Break /* 30 */,	XCF_Edit_Delete /* 31 */,
+   XCF_ChangeStyle /* 30 */,	XCF_Edit_Delete /* 31 */,
    XCF_Edit_Insert /* 32 */,	XCF_Edit_Append /* 33 */,
    XCF_Edit_Next /* 34 */,	XCF_Attach /* 35 */,
    XCF_Next_Library /* 36 */,	XCF_Library_Directory /* 37 */,
@@ -1250,15 +1268,14 @@ enum {
    XCF_Exit /* 88 */,		XCF_Netlist /* 89 */,
    XCF_Swap /* 90 */,		XCF_Pin_Label /* 91 */,
    XCF_Pin_Global /* 92 */,	XCF_Info_Label /* 93 */,
-   XCF_Graphic	/* 94 */,	XCF_Rescale /* 95 */,
-   XCF_SelectBox /* 96 */,	XCF_Connectivity /* 97 */,
-   XCF_Continue_Copy /* 98 */,	XCF_Continue_Element /* 99 */,
-   XCF_Finish_Copy /* 100 */,	XCF_Finish_Element /* 101 */,
-   XCF_Finish /* 102 */,	XCF_Cancel_Last /* 103 */,
-   XCF_Sim /* 104 */,		XCF_SPICE /* 105 */,
-   XCF_PCB /* 106 */,		XCF_SPICEflat /* 107 */,
-   XCF_Color /* 108 */,		XCF_Text_Delete_Param /* 109 */,
-   XCF_Scale /* 110 */,		XCF_ChangeStyle /* 111 */,
+   XCF_Graphic	/* 94 */,	XCF_SelectBox /* 95 */,
+   XCF_Connectivity /* 96 */,	XCF_Continue_Element /* 97 */,
+   XCF_Finish_Element /* 98 */, XCF_Continue_Copy /* 99 */,
+   XCF_Finish_Copy /* 100 */,	XCF_Finish /* 101 */,
+   XCF_Cancel_Last /* 102 */,	XCF_Sim /* 103 */,
+   XCF_SPICE /* 104 */,		XCF_PCB /* 105 */,
+   XCF_SPICEflat /* 106 */,	XCF_Rescale /* 107 */,		
+   XCF_Text_Delete_Param /* 108 */,
    NUM_FUNCTIONS
 };
 
@@ -1340,24 +1357,28 @@ typedef struct _windowdata {
    char		buschar;	/* Character indicating vector notation */
    Boolean	editinplace;
    Boolean	pinpointon;
-   Boolean	attachto;
+   Boolean	pinattach;	/* keep wires attached to pins when moving objinsts */
+#ifndef TCL_WRAPPER
 #ifdef HAVE_XPM
    Boolean	toolbar_on;
+#endif
 #endif
 
    /* buffers and associated variables */
    XPoint	save, origin;
    short	selects;
    short	*selectlist;
+   short	editpart;
+   short	attachto;
    short	lastlibrary;
+   short	textpos;
+   short	textend;
    objinstptr	topinstance;
    objectptr	editstack;
    Matrixptr	MatStack;
    pushlistptr	stack;
    pushlistptr	hierstack;
    short	event_mode;
-   short	editcycle;
-   short	editpart, editsubpart;
    char		*lastbackground;
    Cursor	*defaultcursor;
 } XCWindowData;
@@ -1385,17 +1406,27 @@ typedef struct _undostack {
 /* the library page, and not be destroyed on every call to "composelib"	*/
 
 typedef struct {
-   char		*filename;	/* Last file loaded into the library */
-   char		flags;		/* Flags for library page (changed, read-only) */
    short	number;
    objectptr	*library;
    liblistptr	instlist;	/* List of instances */
 } Library;
 
+typedef struct _Technology *TechPtr;
+
+typedef struct _Technology {
+   u_char	flags;		/* Flags for library page (changed, read-only) */
+   char		*technology;	/* Namespace name (without the "::") */
+   char		*filename;	/* Library file associated with technology */
+   TechPtr 	next;		/* Linked list */
+} Technology;
+
 /* Known flags for library pages */
 
 #define LIBRARY_CHANGED		0x01
 #define LIBRARY_READONLY	0x02
+#define LIBRARY_REPLACE		0x04	/* Replace instances when reading */
+#define LIBRARY_REPLACE_TEMP	0x08	/* Temporary store */
+#define LIBRARY_USED		0x10	/* Temporary marker flag */
 
 /*----------------------------------------------------------------------*/
 /* A convenient structure for holding all the object lists		*/
@@ -1409,10 +1440,11 @@ typedef struct {
    Boolean	retain_backup;
    xcIntervalId	timeout_id;
    int		save_interval;
-   short	do_replace;
    Boolean	filefilter;	/* Is the file list filtered? */
    Boolean	hold;		/* allow HOLD modifiers on buttons */
+   Boolean	showtech;	/* Write technology names in library */
    u_short	new_changes;
+   char		suspend;	/* suspend graphics updates if TRUE */
    short	numlibs;
    short	pages;
    Pagedata	**pagelist;
@@ -1420,6 +1452,7 @@ typedef struct {
    Undoptr	redostack;	/* actions to redo */
    Library	fontlib;
    Library	*userlibs;
+   TechPtr 	technologies;
    objinstptr   *libtop;
    Imagedata	*imagelist;
    short	images;
