@@ -67,7 +67,7 @@
 /*----------------------------------------------------------------------*/
 
 short	 popups;      /* total number of popup widgets on the screen */
-Boolean	 pressmode;   /* Whether we are in a press & hold state */
+int	 pressmode;   /* Whether we are in a press & hold state */
 
 xcWidget   top;
 xcWidget   message1, message2, message3, toolbar, overlay = NULL;
@@ -93,7 +93,6 @@ extern xcIntervalId printtime_id;
 extern short beeper;
 extern short fontcount;
 extern fontinfo *fonts;
-extern short textpos;
 extern short help_up;
 extern menustruct TopButtons[];
 #ifdef HAVE_XPM
@@ -1019,7 +1018,10 @@ void outputpopup(xcWidget button, caddr_t clientdata, caddr_t calldata)
    sprintf(statics[5], "Orientation:");
    sprintf(statics[6], "Mode:");
 
-   sprintf(edit[0], "%s", curpage->filename);
+   if (curpage->filename)
+      sprintf(edit[0], "%s", curpage->filename);
+   else
+      sprintf(edit[0], "Page %d", areawin->page + 1);
    sprintf(edit[1], "%s", topobject->name);
 
    /* recompute bounding box and auto-scale, if set */
@@ -2024,7 +2026,7 @@ int main(int argc, char **argv)
 
    for (k = argc - 1; k > 0; k--) {
       if (!strncmp(argv[k], "-2", 2)) {
-	 pressmode = True;	/* 2-button mouse indicator */
+	 pressmode = 1;		/* 2-button mouse indicator */
 	 break;
       }
    }
@@ -2048,7 +2050,7 @@ int main(int argc, char **argv)
 #endif
 
    loadrcfile();
-   pressmode = False;	/* Done using this to mark 2-button mouse mode */
+   pressmode = 0;	/* Done using this to mark 2-button mouse mode */
 
    composelib(PAGELIB);	/* make sure we have a valid page list */
    composelib(LIBLIB);	/* and library directory */
